@@ -18,6 +18,7 @@ import oauth_dropins.facebook
 from oauth_dropins.webutil import util
 from oauth_dropins.webutil.util import json_dumps, json_loads
 
+from . import as1
 from . import source
 
 logger = logging.getLogger(__name__)
@@ -89,7 +90,6 @@ MAX_IDS = 50  # for the ids query param
 
 M_HTML_BASE_URL = 'https://mbasic.facebook.com/'
 M_HTML_TIMELINE_URL = '%s?v=timeline'
-M_HTML_POST_URL = 'story.php?story_fbid=%s&id=%s'
 M_HTML_REACTIONS_URL = 'ufi/reaction/profile/browser/?ft_ent_identifier=%s'
 
 # Use a modern browser user agent so that we get modern HTML tags like article
@@ -1413,7 +1413,7 @@ class Facebook(source.Source):
     })
 
     if rsvps:
-      self.add_rsvps_to_event(
+      as1.add_rsvps_to_event(
         obj, [self.rsvp_to_object(r, event=event) for r in rsvps])
 
     # de-dupe the event's RSVPs by (user) id. RSVP_FIELDS is ordered by
@@ -1424,7 +1424,7 @@ class Facebook(source.Source):
       for rsvp in event.get(field, {}).get('data', []):
         rsvp = self.rsvp_to_object(rsvp, type=field, event=event)
         id_to_rsvp[rsvp['id']] = rsvp
-    self.add_rsvps_to_event(obj, id_to_rsvp.values())
+    as1.add_rsvps_to_event(obj, id_to_rsvp.values())
 
     return self.postprocess_object(obj)
 
@@ -1762,7 +1762,7 @@ class Facebook(source.Source):
       5 July at 21:50
 
     Args:
-      tag: BeautifulSoup Tag
+      tag: :class:`bs4.BeautifulSoup` or :class:`bs4.Tag`
     """
     if not tag:
       return None
@@ -1793,10 +1793,10 @@ class Facebook(source.Source):
     if not (self.cookie_c_user and self.cookie_xs):
       raise NotImplementedError('Scraping requires c_user and xs cookies.')
 
-    def get(url, *params):
+    def get(url, *params, allow_redirects=False):
       url = urllib.parse.urljoin(M_HTML_BASE_URL, url % params)
       cookie = f'c_user={self.cookie_c_user}; xs={self.cookie_xs}'
-      resp = util.requests_get(url, allow_redirects=False, headers={
+      resp = util.requests_get(url, allow_redirects=allow_redirects, headers={
         'Cookie': cookie,
         'User-Agent': SCRAPE_USER_AGENT,
       })
@@ -1804,7 +1804,10 @@ class Facebook(source.Source):
       return resp
 
     if activity_id:
-      resp = get(M_HTML_POST_URL, activity_id, user_id)
+      # permalinks with classic ids now redirect to URLs with pfbid ids
+      # https://about.fb.com/news/2022/09/deterring-scraping-by-protecting-facebook-identifiers/
+
+      resp = get(activity_id, allow_redirects=True)
       activities = [self.scraped_to_activity(resp.text, **kwargs)[0]]
     else:
       resp = get(M_HTML_TIMELINE_URL, user_id)
@@ -1815,7 +1818,7 @@ class Facebook(source.Source):
         fbids = [a['fb_id'] for a in activities]
         activities = []
         for id in fbids:
-          resp = get(M_HTML_POST_URL, id, user_id)
+          resp = get(id)
           activities.append(self.scraped_to_activity(resp.text)[0])
 
     if fetch_likes:
@@ -1849,15 +1852,29 @@ class Facebook(source.Source):
         continue
 
       header = post.find('header')
-      if header and 'Suggested for you' in header.stripped_strings:
-        logger.debug('Skipping "Suggested for you"')
+      if header and ('Suggested for you' in header.stripped_strings or
+                     'is with' in header.stripped_strings):
+        logger.debug(f'Skipping {header.stripped_strings}')
         continue
 
-      url = self._sanitize_url(urllib.parse.urljoin(self.BASE_URL, permalink['href']))
-      query = urllib.parse.urlparse(url).query
-      parsed = urllib.parse.parse_qs(query)
-      post_id = parsed['story_fbid'][0]
-      owner_id = parsed['id'][0]
+      post_id, owner_id = self._extract_scraped_ids(post)
+      if not post_id or not owner_id:
+        url = urllib.parse.urljoin(self.BASE_URL, permalink['href'])
+        query = urllib.parse.urlparse(url).query
+        parsed = urllib.parse.parse_qs(query)
+        # story_fbid stopped being useful in May 2022, it switched to an opaque
+        # token that changes regularly, even for the same post.
+        # https://github.com/snarfed/facebook-atom/issues/27
+        ft = util.get_first(parsed, '_ft_') or ''
+        for elem in ft.split(':'):
+          if elem.startswith('top_level_post_id.') or elem.startswith('mf_objid.'):
+            post_id = elem.split('.')[1]
+            if post_id:
+              break
+        else:
+          post_id = util.get_first(parsed, 'story_fbid')
+
+        owner_id = parsed['id'][0]
 
       author = self._m_html_author(post)
       if not author:
@@ -1923,6 +1940,7 @@ class Facebook(source.Source):
           if util.is_int(count_text):
             reactions_count = int(count_text)
 
+      url = self._sanitize_url(urllib.parse.urljoin(self.BASE_URL, f'/{post_id}'))
       activities.append({
         'objectType': 'activity',
         'verb': 'post',
@@ -2024,30 +2042,15 @@ class Facebook(source.Source):
             }
           })
 
-    # TODO: unify with scraped_to_activities
-    data_ft_div = soup.find(attrs={
-      'data-ft': re.compile('mf_story_key|top_level_post_id'),
-    })
-    comment_form = soup.find('form', action=re.compile('^/a/comment.php'))
-    if data_ft_div:
-      data_ft = json_loads(html.unescape(data_ft_div['data-ft']))
-      post_id = data_ft.get('mf_story_key') or data_ft.get('top_level_post_id')
-      owner_id = str(data_ft.get('content_owner_id_new')
-                     if 'mf_story_key' in data_ft  # this post is from a user
-                     else data_ft.get('page_id'))  # this post is from a group
-    elif comment_form:
-      query = urllib.parse.urlparse(comment_form['action']).query
-      parsed = urllib.parse.parse_qs(query)
-      post_id = parsed['ft_ent_identifier'][0]
-      owner_id = parsed['av'][0]
-    else:
+    post_id, owner_id = self._extract_scraped_ids(soup)
+    if not post_id or not owner_id:
       return activity, None
 
     # numeric ids
     ids = {
       'id': self.tag_uri(post_id),
       'fb_id': post_id,
-      'url': f'{self.BASE_URL}story.php?story_fbid={post_id}&id={owner_id}',
+      'url': f'{self.BASE_URL}{post_id}',
     }
     activity.update(ids)
     activity['object'].update(ids)
@@ -2084,6 +2087,45 @@ class Facebook(source.Source):
 
     return util.trim_nulls(activity), None
 
+  @staticmethod
+  def _extract_scraped_ids(soup):
+    """Tries to scrape post id and owner id out of parsed HTML.
+
+    Background on the time-limited pfbid param and why we don't want to use it
+    in permalinks:
+    https://about.fb.com/news/2022/09/deterring-scraping-by-protecting-facebook-identifiers/
+    Args:
+      soup: :class:`bs4.BeautifulSoup` or :class:`bs4.Tag`
+
+    Returns: (str post id, str owner id) tuple, or (None, None) if not found
+    """
+    post_id = owner_id = None
+
+    post_id_re = re.compile('mf_story_key|top_level_post_id')
+    data_ft_div = (soup if post_id_re.search(soup.get('data-ft', ''))
+                   else soup.find(attrs={'data-ft': post_id_re}))
+
+    if data_ft_div:
+      data_ft = json_loads(html.unescape(data_ft_div['data-ft']))
+      # prefer top_level_post_id, mf_story_key doesn't always work in FB URLs
+      # that we construct, eg https://www.facebook.com/[mf_story_key]
+      post_id = data_ft.get('top_level_post_id') or data_ft.get('mf_story_key')
+      owner_id = str(data_ft.get('content_owner_id_new')
+                     if 'mf_story_key' in data_ft  # this post is from a user
+                     else data_ft.get('page_id'))  # this post is from a group
+
+    if post_id and not post_id.startswith('pfbid'):
+      return post_id, owner_id
+
+    comment_form = soup.find('form', action=re.compile('^/a/comment.php'))
+    if comment_form:
+      query = urllib.parse.urlparse(comment_form['action']).query
+      parsed = urllib.parse.parse_qs(query)
+      post_id = parsed['ft_ent_identifier'][0]
+      owner_id = parsed['av'][0]
+
+    return post_id, owner_id
+
   def merge_scraped_reactions(self, scraped, activity):
     """Converts and merges scraped likes and reactions into an activity.
 
@@ -2127,7 +2169,7 @@ class Facebook(source.Source):
         tag['content'] = REACTION_CONTENT.get(type.upper())
       tags.append(tag)
 
-    source.merge_by_id(activity['object'], 'tags', tags)
+    as1.merge_by_id(activity['object'], 'tags', tags)
     return tags
 
   def scraped_to_actor(self, scraped):
