@@ -17,6 +17,7 @@ from oauth_dropins.webutil import util
 from oauth_dropins.webutil.util import json_dumps, json_loads
 import requests
 
+from . import as1
 from . import source
 
 logger = logging.getLogger(__name__)
@@ -224,9 +225,18 @@ class GitHub(source.Source):
   # https://github.com/shinnn/github-username-regex#readme
   # (this slightly overspecifies; it allows multiple consecutive hyphens and
   # leading/trailing hyphens. oh well.)
-  USER_NAME_RE = re.compile(r'^[A-Za-z0-9-]+$')
+  USER_NAME_RE = re.compile(r'[A-Za-z0-9-]+')
+  USER_URL_RE = re.compile(fr"""
+    (?<!]\(|=['"])  # don't match on Markdown links or HTML attributes
+    \b
+    {re.escape(BASE_URL)}
+    ({USER_NAME_RE.pattern})
+    \b
+    (?![/?_+#@.])  # don't match on URLs that continue past username
+  """, re.VERBOSE)
+
   # https://github.com/moby/moby/issues/679#issuecomment-18307522
-  REPO_NAME_RE = re.compile(r'^[A-Za-z0-9_.-]+$')
+  REPO_NAME_RE = re.compile(r'[A-Za-z0-9_.-]+')
   # https://github.com/Alir3z4/html2text/blob/master/docs/usage.md#available-options
   HTML2TEXT_OPTIONS = {
     'ignore_images': False,
@@ -278,7 +288,7 @@ class GitHub(source.Source):
         'Authorization': f'bearer {self.access_token}',
       })
     resp.raise_for_status()
-    result = json_loads(resp.text)
+    result = resp.json()
 
     errs = result.get('errors')
     if errs:
@@ -301,9 +311,6 @@ class GitHub(source.Source):
     kwargs['headers'] = kwargs.get('headers') or {}
     kwargs['headers'].update({
       'Authorization': f'token {self.access_token}',
-      # enable the beta Reactions API
-      # https://developer.github.com/v3/reactions/
-      'Accept': 'application/vnd.github.squirrel-girl-preview+json',
     })
 
     if data is None:
@@ -312,7 +319,7 @@ class GitHub(source.Source):
       resp = util.requests_post(url, json=data, **kwargs)
     resp.raise_for_status()
 
-    return json_loads(resp.text) if parse_json else resp
+    return resp.json() if parse_json else resp
 
   def get_activities_response(self, user_id=None, group_id=None, app_id=None,
                               activity_id=None, start_index=0, count=0,
@@ -347,15 +354,17 @@ class GitHub(source.Source):
 
     etag_parsed = email.utils.parsedate(etag)
     since = datetime.datetime(*etag_parsed[:6]) if etag_parsed else None
+    issues = []
     activities = []
 
     if activity_id:
+      # single issue
       parts = tuple(activity_id.split(':'))
       if len(parts) != 3:
         raise ValueError('GitHub activity ids must be of the form USER:REPO:ISSUE_OR_PR')
       try:
-        issue = self.rest(REST_ISSUE % parts)
-        activities = [self.issue_to_object(issue)]
+        issues = [self.rest(REST_ISSUE % parts)]
+        activities = [self.issue_to_object(issues[0])]
       except BaseException as e:
         code, body = util.interpret_http_exception(e)
         if util.is_int(code) and int(code) in HTTP_NON_FATAL_CODES:
@@ -364,10 +373,11 @@ class GitHub(source.Source):
           raise
 
     else:
+      # all issues/PRs, based on notifications
       resp = self.rest(REST_NOTIFICATIONS, parse_json=False,
                        headers={'If-Modified-Since': etag} if etag else None)
       etag = resp.headers.get('Last-Modified')
-      notifs = [] if resp.status_code == 304 else json_loads(resp.text)
+      notifs = [] if resp.status_code == 304 else resp.json()
 
       for notif in notifs:
         id = notif.get('id')
@@ -399,26 +409,30 @@ class GitHub(source.Source):
             'alias': '@private' if private else '@public',
           }]
 
-        comments_url = issue.get('comments_url')
-        if fetch_replies and comments_url:
-          if since:
-            comments_url += f'?since={since.isoformat()}' + 'Z'
-          comments = self.rest(comments_url)
-          comment_objs = list(util.trim_nulls(
-            self.comment_to_object(c) for c in comments))
-          obj['replies'] = {
-            'items': comment_objs,
-            'totalItems': len(comment_objs),
-          }
-
-        if fetch_likes:
-          issue_url = issue['url'].replace('pulls', 'issues')
-          reactions = self.rest(issue_url + '/reactions')
-          obj.setdefault('tags', []).extend(
-            self.reaction_to_object(r, obj) for r in reactions)
-
+        issues.append(issue)
         activities.append(obj)
 
+    # add comments and reactions, if requested
+    assert len(issues) == len(activities)
+    for issue, obj in zip(issues, activities):
+      comments_url = issue.get('comments_url')
+      if fetch_replies and comments_url:
+        if since:
+          comments_url += f'?since={since.isoformat()}' + 'Z'
+        comments = self.rest(comments_url)
+        comment_objs = list(util.trim_nulls(
+          self.comment_to_object(c) for c in comments))
+        obj['replies'] = {
+          'items': comment_objs,
+          'totalItems': len(comment_objs),
+        }
+
+      if fetch_likes:
+        issue_url = issue['url'].replace('pulls', 'issues')
+        reactions = self.rest(issue_url + '/reactions')
+        obj.setdefault('tags', []).extend(
+          self.reaction_to_object(r, obj) for r in reactions)
+
     response = self.make_activities_base_response(util.trim_nulls(activities))
     response['etag'] = etag
     return response
@@ -537,9 +551,9 @@ class GitHub(source.Source):
     """
     assert preview in (False, True)
 
-    type = source.object_type(obj)
+    type = as1.object_type(obj)
     if type and type not in ('issue', 'comment', 'activity', 'note', 'article',
-                             'like', 'tag'):
+                             'like', 'favorite', 'tag'):
       return source.creation_result(
         abort=False, error_plain=f'Cannot publish {type} to GitHub')
 
@@ -553,6 +567,11 @@ class GitHub(source.Source):
     content = orig_content = replace_placeholders(html.escape(
       self._content_for_create(obj, ignore_formatting=ignore_formatting),
       quote=False))
+
+    # convert GitHub user profile URLs to @-mentions,
+    # eg https://github.com/snarfed to @snarfed
+    content = self.USER_URL_RE.sub(r'@\1', content)
+
     url = obj.get('url')
     if include_link == source.INCLUDE_LINK and url:
       content += f'\n\n(Originally published at: {url})'
@@ -640,7 +659,7 @@ class GitHub(source.Source):
           except ValueError as e:
             return source.creation_result(abort=True, error_plain=str(e))
 
-    elif type == 'like':  # star
+    elif type in ('like', 'favorite'):  # star
       if not (len(path) == 2 or (len(path) == 3 and path[2] == 'issues')):
         return source.creation_result(
           abort=True, error_plain='GitHub like requires in-reply-to repo URL.')
