@@ -19,6 +19,7 @@ from oauth_dropins.webutil import util
 from oauth_dropins.webutil.util import json_dumps, json_loads
 import requests
 
+from . import as1
 from . import source
 
 logger = logging.getLogger(__name__)
@@ -52,7 +53,13 @@ HTML_DATA_RE = re.compile(r"""
   window\.(_sharedData\ =|__additionalDataLoaded\('[^']+',)\ *
   (.+?)
   \)?;</script>""", re.VERBOSE)
-# duplicated in bridgy/browser-extension/instagram.js
+HTML_DEFINES_RE = re.compile(r"""
+  handleWithCustomApplyEach\(ScheduledApplyEach, *
+  (.+?)
+  \);}\);}""", re.VERBOSE)
+
+# duplicated in bridgy/browser-extension/instagram.js and
+# instagram-atom/browser-extension/instagram.js
 HEADERS = {
   'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:96.0) Gecko/20100101 Firefox/96.0',
   'X-IG-App-ID': '936619743392459',  # desktop web
@@ -338,16 +345,9 @@ class Instagram(source.Source):
                                                    count=count)
 
     if fetch_extras:
-      # batch get cached counts of comments and likes for all activities
-      cached = {}
-      # don't update the cache until the end, in case we hit an error before
-      cache_updates = {}
-      if cache is not None:
-        keys = []
-        for activity in activities:
-          _, id = util.parse_tag_uri(activity['id'])
-          keys.extend(['AIL ' + id, 'AIC ' + id])
-        cached = cache.get_multi(keys)
+      if cache is None:
+        # for convenience, throwaway object just for this method
+        cache = {}
 
       for i, activity in enumerate(activities):
         obj = activity['object']
@@ -357,8 +357,8 @@ class Instagram(source.Source):
         likes_key = f'AIL {id}'
         comments_key = f'AIC {id}'
 
-        if (likes and likes != cached.get(likes_key) or
-            comments and comments != cached.get(comments_key)):
+        if (likes and likes != cache.get(likes_key) or
+            comments and comments != cache.get(comments_key)):
           if not activity_id and not shortcode:
             url = activity['url'].replace(self.BASE_URL, HTML_BASE_URL)
             resp = util.requests_get(url, **get_kwargs)
@@ -369,10 +369,7 @@ class Instagram(source.Source):
             resp.text, cookie=cookie, count=count, fetch_extras=fetch_extras)
           if full_activity:
             activities[i] = full_activity[0]
-            cache_updates.update({likes_key: likes, comments_key: comments})
-
-      if cache_updates and cache is not None:
-        cache.set_multi(cache_updates)
+            cache.update({likes_key: likes, comments_key: comments})
 
     resp = self.make_activities_base_response(activities)
     resp['actor'] = actor
@@ -793,8 +790,7 @@ class Instagram(source.Source):
         'alias': '@private' if private else '@public',
       }]
 
-
-  def scraped_to_activities(self, html, cookie=None, count=None,
+  def scraped_to_activities(self, input, cookie=None, count=None,
                             fetch_extras=False):
     """Converts scraped Instagram HTML to ActivityStreams activities.
 
@@ -803,9 +799,11 @@ class Instagram(source.Source):
     * a user's feed, eg https://www.instagram.com/ while logged in
     * a user's profile, eg https://www.instagram.com/snarfed/
     * a photo or video, eg https://www.instagram.com/p/BBWCSrfFZAk/
+    * serialized JSON from the API for a feed, profile, or post, eg
+      https://i.instagram.com/api/v1/feed/timeline/
 
     Args:
-      html: unicode string
+      input: unicode string containing either HTML or JSON
       cookie: string, optional sessionid cookie to be used for subsequent HTTP
         fetches, if necessary.
       count: integer, number of activities to return, None for all
@@ -815,23 +813,78 @@ class Instagram(source.Source):
       tuple, ([ActivityStreams activities], ActivityStreams viewer actor)
     """
     cookie = cookie or self.cookie
-    # extract JSON data blob
-    # (can also get just this JSON by adding ?__a=1 to any IG URL.)
-    matches = HTML_DATA_RE.findall(html)
-    if not matches:
-      # Instagram sometimes returns 200 with incomplete HTML. often it stops at
-      # the end of one of the <style> tags inside <head>. not sure why.
-      logger.warning('JSON script tag not found!')
-      return [], None
 
+    # sniff JSON input
+    if input and input[0] in ('{', '['):
+      try:
+        input = json_loads(input)
+      except ValueError:
+        return [], None
+      return self.scraped_json_to_activities(
+        input, cookie=cookie, count=count, fetch_extras=fetch_extras)
+
+    # extract JSON data blob from HTML
+    matches = HTML_DATA_RE.findall(input)
+    if matches:
+      data = [util.trim_nulls(json_loads(match[1])) for match in matches]
+      activities, actor = self.scraped_json_to_activities(
+        data, cookie=cookie, count=count, fetch_extras=fetch_extras)
+      if activities or actor:
+        return activities, actor
+
+    match = HTML_DEFINES_RE.search(input)
+    if match:
+      data = json_loads(match[1])
+      for define in data.get('define', []):
+        if len(define) >= 3 and define[0] == 'XIGSharedData':
+          xigshared = define[2].get('raw', '{}')
+          activities, actor = self.scraped_json_to_activities(
+            json_loads(xigshared), cookie=cookie, count=count, fetch_extras=fetch_extras)
+          if activities or actor:
+            return activities, actor
+
+    # As of 2018-02-15, embedded JSON in logged in https://www.instagram.com/
+    # sometimes has no useful data. Need to do a second header link fetch.
+    soup = util.parse_html(input)
+    link = soup.find('link', href=HTML_PRELOAD_RE)
+    if link:
+      url = urllib.parse.urljoin(HTML_BASE_URL, link['href'])
+      return self.scraped_json_to_activities(
+        self._scrape_json(url, cookie=cookie), cookie=cookie, count=count,
+        fetch_extras=fetch_extras)
+
+    logger.warning("Couldn't find JSON data in scraped input!")
+    return [], None
+
+  def scraped_json_to_activities(self, input, cookie=None, count=None,
+                                 fetch_extras=False):
+    """Converts scraped Instagram JSON to ActivityStreams activities.
+
+    The input JSON may be from:
+
+    * a user's profile, eg
+      https://i.instagram.com/api/v1/users/web_profile_info/?username=...
+
+    Args:
+      input: dict or sequence of dicts, Instagram JSON object(s)
+      cookie: string, optional sessionid cookie to be used for subsequent HTTP
+        fetches, if necessary.
+      count: integer, number of activities to return, None for all
+      fetch_extras: whether to make extra HTTP fetches to get likes, etc.
+
+    Returns:
+      tuple, ([ActivityStreams activities], ActivityStreams viewer actor)
+    """
     # find media
     medias = []
     feed_v2_items = []
     profile_user = None
     viewer_user = None
 
-    for match in matches:
-      data = util.trim_nulls(json_loads(match[1]))
+    if not isinstance(input, (list, tuple, set)):
+      input = [input]
+
+    for data in input:
       entry_data = data.get('entry_data', {})
 
       # home page ie news feed
@@ -843,16 +896,19 @@ class Instagram(source.Source):
                       ('GraphSuggestedUserFeedUnit',))
 
       # feed v2
-      feed_v2_items.extend(data.get('feed_items', []))
-      feed_v2_items.extend(data.get('items', []))
+      feed_v2_items.extend(data.get('feed_items') or [])
+      feed_v2_items.extend(data.get('items') or [])
 
-      if 'user' in data:
-        edges = data['user'].get('edge_web_feed_timeline', {}).get('edges', [])
-        medias.extend(e.get('node') for e in edges)
+      user = (data.get('data', {}) or data).get('user', {})
+      edges = user.get('edge_web_feed_timeline', {}).get('edges', [])
+      medias.extend(e.get('node') for e in edges)
 
       # user profiles
-      for page in entry_data.get('ProfilePage', []):
-        profile_user = page.get('graphql', {}).get('user', {})
+      profile_users = [page.get('graphql', {}).get('user', {})
+                       for page in entry_data.get('ProfilePage', [])]
+      if user:
+        profile_users.append(user)
+      for profile_user in profile_users:
         medias.extend(edge['node'] for edge in
           profile_user.get('edge_owner_to_timeline_media', {}).get('edges', [])
           if edge.get('node'))
@@ -866,18 +922,6 @@ class Instagram(source.Source):
         if media:
           medias.append(media)
 
-    if not medias and not feed_v2_items:
-      # As of 2018-02-15, embedded JSON in logged in https://www.instagram.com/
-      # no longer has any useful data. Need to do a second header link fetch.
-      soup = util.parse_html(html)
-      link = soup.find('link', href=HTML_PRELOAD_RE)
-      if link:
-        url = urllib.parse.urljoin(HTML_BASE_URL, link['href'])
-        data = self._scrape_json(url, cookie=cookie)
-        edges = data.get('data', {}).get('user', {})\
-                    .get('edge_web_feed_timeline', {}).get('edges', [])
-        medias = [e.get('node') for e in edges]
-
     if count:
       medias = medias[:count]
 
@@ -900,6 +944,8 @@ class Instagram(source.Source):
       media = item.get('media_or_ad') or item
       if media and (not count or len(activities) < count):
         activity = self._feed_v2_item_to_activity(media)
+        if not activity:
+          continue
         self.merge_scraped_comments(media, activity)
         self.merge_scraped_reactions({'data': {'shortcode_media': media}}, activity)
 
@@ -990,7 +1036,7 @@ class Instagram(source.Source):
 
     obj_replies = obj.setdefault('replies', {})
     if replies:
-      source.merge_by_id(obj_replies, 'items', replies)
+      as1.merge_by_id(obj_replies, 'items', replies)
 
     obj_replies['totalItems'] = max(scraped.get('comment_count', 0),
                                     obj_replies.get('totalItems', 0),
@@ -1026,10 +1072,11 @@ class Instagram(source.Source):
       media_url = self.media_url(shortcode) if shortcode else obj.get('url')
       likers = [l.get('node', {}) for l in
                 media.get('edge_liked_by', {}).get('edges', [])]  # v1
-      likers.extend(media.get('likers', []))  # v2
+      likers.extend(media.get('likers', []) +  # v2
+                    media.get('facepile_top_likers', []))
       like_tags = util.trim_nulls(
         [self.like_to_object(l, id, media_url) for l in likers])
-      source.merge_by_id(obj, 'tags', like_tags)
+      as1.merge_by_id(obj, 'tags', like_tags)
       return like_tags
 
     return []
@@ -1037,17 +1084,18 @@ class Instagram(source.Source):
   @staticmethod
   def _scrape_json(url, cookie=None):
     """Fetches and returns JSON from www.instagram.com."""
-    headers = {}
-    if cookie:
-      if not cookie.startswith('sessionid='):
-        cookie = 'sessionid=' + cookie
-      headers = {'Cookie': cookie, **HEADERS}
+    if not cookie:
+      return {}
+
+    if not cookie.startswith('sessionid='):
+      cookie = 'sessionid=' + cookie
+    headers = {'Cookie': cookie, **HEADERS}
 
     resp = util.requests_get(url, allow_redirects=False, headers=headers)
     resp.raise_for_status()
 
     try:
-      return json_loads(resp.text)
+      return resp.json()
     except ValueError:
       msg = f"Couldn't decode response as JSON:\n{resp.text}"
       logger.error(msg, exc_info=True)
@@ -1183,17 +1231,19 @@ class Instagram(source.Source):
       media: dict, item from a feed_v2 JSON
 
     Returns:
-      dict, ActivityStreams activity
+      dict, ActivityStreams activity, or None
     """
     user = item.get('user') or {}
     actor = self._feed_v2_user_to_actor(user)
 
     item_pk = item.get("pk")
     user_pk = user.get("pk")
+    if not item_pk and not user_pk:
+      return None
     obj_id = self.tag_uri(f'{item_pk}_{user_pk}' if user_pk else item_pk)
 
     media_url = self.media_url(item.get('code'))
-    caption = item.get('caption', {})
+    caption = item.get('caption') or {}
 
     # media
     attachments = []
