@@ -13,8 +13,9 @@ import re
 import urllib.parse
 
 from oauth_dropins.webutil import util
+from oauth_dropins.webutil.testutil import requests_response
 from oauth_dropins.webutil.util import json_dumps, json_loads
-import requests
+from requests import HTTPError, RequestException
 
 from . import source
 
@@ -141,16 +142,23 @@ class Mastodon(source.Source):
 
     if not return_json:
       return resp
+
     if fn == util.requests_delete:
       return {}
-    else:
-      return json_loads(resp.text)
+
+    content_type = resp.headers.get('Content-Type')
+    if not content_type or content_type.split(';')[0] != 'application/json':
+      logger.warning(f'Converting {resp.status_code} to 502 due to Content-Type {content_type} instead of application/json')
+      resp.status_code = 502
+      raise HTTPError(response=resp)
+
+    return resp.json()
 
   @classmethod
   def embed_post(cls, obj):
     """Returns the HTML string for embedding a toot.
 
-    https://dev.twitter.com/docs/embedded-tweets
+    https://docs.joinmastodon.org/methods/oembed/
 
     Args:
       obj: AS1 dict with at least url, and optionally also content.
@@ -206,13 +214,9 @@ class Mastodon(source.Source):
 
     activities = []
 
-    # batch get memcached counts of likes, boosts, etc
-    cached = {}
-    if cache is not None:
-      keys = itertools.product(('AMRE', 'AMF', 'AMRB'), [s['id'] for s in statuses])
-      cached = cache.get_multi(f'{prefix} {id}' for prefix, id in keys)
-    # only update the cache at the end, in case we hit an error before then
-    cache_updates = {}
+    if cache is None:
+      # for convenience, throwaway object just for this method
+      cache = {}
 
     # fetch extras if necessary
     for status in statuses[start_index:]:
@@ -227,26 +231,26 @@ class Mastodon(source.Source):
 
       obj = activity['object']
       count = status.get('replies_count')
-      if fetch_replies and count and count != cached.get('AMRE ' + id):
+      if fetch_replies and count and count != cache.get('AMRE ' + id):
         context = self._get(API_CONTEXT % id)
         obj['replies'] = {
           'items': [self.status_to_activity(reply)
                     for reply in context.get('descendants', [])]
         }
-        cache_updates['AMRE ' + id] = count
+        cache['AMRE ' + id] = count
 
       tags = obj.setdefault('tags', [])
       count = status.get('favourites_count')
-      if fetch_likes and count and count != cached.get('AMF ' + id):
+      if fetch_likes and count and count != cache.get('AMF ' + id):
         likers = self._get(API_FAVORITED_BY % id)
         tags.extend(self._make_like(status, l) for l in likers)
-        cache_updates['AMF ' + id] = count
+        cache['AMF ' + id] = count
 
       count = status.get('reblogs_count')
-      if fetch_shares and count and count != cached.get('AMRB ' + id):
+      if fetch_shares and count and count != cache.get('AMRB ' + id):
         sharers = self._get(API_REBLOGGED_BY % id)
         tags.extend(self._make_share(status, s) for s in sharers)
-        cache_updates['AMRB ' + id] = count
+        cache['AMRB ' + id] = count
 
     if fetch_mentions:
       # https://docs.joinmastodon.org/methods/notifications/
@@ -257,8 +261,6 @@ class Mastodon(source.Source):
                         if n.get('status') and n.get('type') == 'mention')
 
     resp = self.make_activities_base_response(util.trim_nulls(activities))
-    if cache_updates and cache is not None:
-      cache.set_multi(cache_updates)
     return resp
 
   def get_actor(self, user_id=None):
@@ -600,13 +602,6 @@ class Mastodon(source.Source):
           error_plain='No content text found.',
           error_html='No content text found.')
 
-    post_label = f"{self.NAME} {self.TYPE_LABELS['post']}"
-    if is_reply and not base_url:
-      return source.creation_result(
-        abort=True,
-        error_plain=f'Could not find a {post_label} to reply to.',
-        error_html=f'Could not find a {post_label} to <a href="http://indiewebcamp.com/reply">reply to</a>. Check that your post has the right <a href="http://indiewebcamp.com/comment">in-reply-to</a> link.')
-
     # truncate and ellipsize content if necessary
     # TODO: don't count domains in remote mentions.
     # https://docs.joinmastodon.org/user/posting/#text
@@ -628,8 +623,10 @@ class Mastodon(source.Source):
     preview_content = HASHTAG_RE.sub(r'\1<a href="%s/\2">#\2</a>' % tags_url,
                                      preview_content)
 
+    post_label = f"{self.NAME} {self.TYPE_LABELS['post']}"
+
     # switch on activity type
-    if type == 'activity' and verb == 'like':
+    if type == 'activity' and verb in ('like', 'favorite'):
       if not base_url:
         return source.creation_result(
           abort=True,
@@ -662,7 +659,7 @@ class Mastodon(source.Source):
           (type == 'activity' and verb == 'post')):  # probably a bookmark
       data = {'status': content}
 
-      if is_reply:
+      if is_reply and base_url:
         preview_description += f"add a <span class=\"verb\">{self.TYPE_LABELS['comment']}</span> to <a href=\"{base_url}\">this {self.TYPE_LABELS['post']}</a>: {self.embed_post(base_obj)}"
         data['in_reply_to_id'] = base_id
       else:
@@ -738,7 +735,7 @@ class Mastodon(source.Source):
         # nope; try mastodon's search API
         try:
           results = self._get(API_SEARCH, params={'q': url, 'resolve': True})
-        except requests.RequestException:
+        except RequestException:
           logger.info(f"{field} URL {url} doesn't look like Mastodon:")
           continue
 
@@ -829,7 +826,7 @@ class Mastodon(source.Source):
     url = API_BLOCKS
     while True:
       resp = self._get(url, return_json=False)
-      ids.extend(util.trim_nulls([rel.get('id') for rel in json_loads(resp.text)]))
+      ids.extend(util.trim_nulls([rel.get('id') for rel in resp.json()]))
       url = resp.links.get('next', {}).get('url')
       if not url:
         break
