@@ -25,6 +25,7 @@ from oauth_dropins.webutil.util import (
   uniquify,
 )
 
+from . import as1
 from . import source
 
 logger = logging.getLogger(__name__)
@@ -154,7 +155,7 @@ def _activity_or_object(activity):
 
   Used in :func:`activity_to_json()` and :func:`activities_to_html()`.
   """
-  if activity.get('object') and activity.get('verb') not in source.VERBS_WITH_OBJECT:
+  if activity.get('object') and activity.get('verb') not in as1.VERBS_WITH_OBJECT:
     return activity['object']
 
   return activity
@@ -181,19 +182,20 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   if not obj or not isinstance(obj, dict):
     return {}
 
-  obj_type = source.object_type(obj) or default_object_type
+  obj_type = as1.object_type(obj) or default_object_type
   # if the activity type is a post, then it's really just a conduit
   # for the object. for other verbs, the activity itself is the
   # interesting thing
   if obj_type == 'post':
     primary = obj.get('object', {})
-    obj_type = source.object_type(primary) or default_object_type
+    obj_type = as1.object_type(primary) or default_object_type
   else:
     primary = obj
 
   # TODO: extract snippet
   name = primary.get('displayName', primary.get('title'))
   summary = primary.get('summary')
+  note = primary.get('note')
   author = obj.get('author', obj.get('actor', {}))
 
   in_reply_tos = obj.get('inReplyTo') or []
@@ -242,8 +244,8 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
       'numeric-id': [obj.get('numeric_id') or ''],
       'name': [name],
       'nickname': [obj.get('username') or ''],
-      'summary': [summary],
-      'url': (list(object_urls(obj) or object_urls(primary)) +
+      ('note' if obj_type == 'person' else 'summary'): [summary],
+      'url': (list(as1.object_urls(obj) or as1.object_urls(primary)) +
               obj.get('upstreamDuplicates', [])),
       # photo is special cased below, to handle alt
       'video': dedupe_urls(get_urls(attachments, 'video', 'stream') +
@@ -278,23 +280,25 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   # content. emulate e- vs p- microformats2 parsing: e- if there are HTML tags,
   # otherwise p-.
   # https://indiewebcamp.com/note#Indieweb_whitespace_thinking
-  text = xml.sax.saxutils.unescape(primary.get('content') or '')
-  html = render_content(primary, include_location=False,
-                        synthesize_content=synthesize_content)
-  if '<' in html:
-    ret['properties']['content'] = [{'value': text, 'html': html}]
+  text = html.unescape(primary.get('content') or '')
+  rendered = render_content(primary, include_location=False,
+                            synthesize_content=synthesize_content)
+  if '<' in rendered:
+    ret['properties']['content'] = [{'value': text, 'html': rendered}]
   else:
     ret['properties']['content'] = [text]
 
   # photos, including alt text
   photo_urls = set()
   ret['properties']['photo'] = []
-  for image in get_list(attachments, 'image') + [primary]:
-    for url in get_urls(image, 'image'):
-      if url and url not in photo_urls:
-        photo_urls.add(url)
-        name = get_first(image, 'image', {}).get('displayName')
-        ret['properties']['photo'].append({'value': url, 'alt': name} if name else url)
+  for img in get_list(attachments, 'image') + get_list(primary, 'image'):
+    if img.get('image'):
+      img = get_first(img, 'image')
+    url = get_url(img)
+    if url and url not in photo_urls:
+      photo_urls.add(url)
+      name = img.get('displayName')
+      ret['properties']['photo'].append({'value': url, 'alt': name} if name else url)
 
   # hashtags and person tags
   if obj_type == 'tag':
@@ -343,7 +347,7 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
       # received likes and reposts
       ret['properties'][prop] = [
         object_to_json(t, trim_nulls=False, entry_class='h-cite')
-        for t in tags if source.object_type(t) == type]
+        for t in tags if as1.object_type(t) == type]
 
   # bookmarks
   if obj_type == 'bookmark':
@@ -500,7 +504,7 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
     'endTime': prop.get('end'),
     'displayName': get_text(prop.get('name')),
     'username': prop.get('nickname'),
-    'summary': get_text(prop.get('summary')),
+    'summary': get_text(prop.get('summary') or prop.get('note')),
     'content': get_html(prop.get('content')),
     'url': urls[0] if urls else None,
     'urls': [{'value': u} for u in urls] if urls and len(urls) > 1 else None,
@@ -517,8 +521,9 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
 
   # images, including alt text
   photo_urls = set()
+  featured = props.get('featured', [])
   obj['image'] = []
-  for photo in props.get('photo', []) + props.get('featured', []):
+  for photo in props.get('photo', []) + featured:
     url = photo
     alt = None
     if isinstance(photo, dict):
@@ -527,7 +532,11 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
       alt = get_first(photo, 'alt')
     if url and url not in photo_urls and is_absolute(url):
       photo_urls.add(url)
-      obj['image'].append({'url': url, 'displayName': alt})
+      obj['image'].append({
+        'url': url,
+        'displayName': alt,
+        'objectType': 'featured' if photo in featured else None,
+      })
 
   # mf2util uses the indieweb/mf2 location algorithm to collect location properties.
   interpreted = mf2util.interpret({'items': [mf2]}, None)
@@ -609,26 +618,30 @@ def html_to_activities(html, url=None, actor=None, id=None):
   return activities
 
 
-def activities_to_html(activities):
+def activities_to_html(activities, extra='', body_class=''):
   """Converts ActivityStreams activities to a microformats2 HTML h-feed.
 
   Args:
     obj: dict, a decoded JSON ActivityStreams object
+    extra: str, extra HTML to be included inside the body tag, at the top
+    body_class: str, included as the body tag's class attribute
 
   Returns:
     string, the content field in obj with the tags in the tags field
     converted to links if they have startIndex and length, otherwise added to
     the end.
   """
-  return """\
+  html = '\n'.join(object_to_html(_activity_or_object(a)) for a in activities)
+  return f"""\
 <!DOCTYPE html>
 <html>
 <head><meta charset="utf-8"></head>
-<body>
-%s
+<body class="{body_class}">
+{extra}
+{html}
 </body>
 </html>
-""" % '\n'.join(object_to_html(_activity_or_object(a)) for a in activities)
+"""
 
 
 def object_to_html(obj, parent_props=None, synthesize_content=True):
@@ -874,7 +887,7 @@ def render_content(obj, include_location=True, synthesize_content=True,
   Returns:
     string, rendered HTML
   """
-  obj_type = source.object_type(obj)
+  obj_type = as1.object_type(obj)
   content = obj.get('content') or ''
 
   # extract tags. preserve order but de-dupe, ie don't include a tag more than
@@ -891,7 +904,7 @@ def render_content(obj, include_location=True, synthesize_content=True,
     if 'startIndex' in t and 'length' in t and 'url' in t:
       mentions.append(t)
     else:
-      tags.setdefault(source.object_type(t), []).append(t)
+      tags.setdefault(as1.object_type(t), []).append(t)
 
   # linkify embedded mention tags inside content.
   if mentions:
@@ -1118,7 +1131,7 @@ def tags_to_html(tags, classname, visible=True):
       name = tag['displayName']
     # loop through individually instead of using update() so that order is
     # preserved.
-    for url in object_urls(tag):
+    for url in as1.object_urls(tag):
       urls[url, name] = None
 
   return ''.join('\n<a class="%s" %shref="%s">%s</a>' %
@@ -1126,15 +1139,6 @@ def tags_to_html(tags, classname, visible=True):
                  for url, name in sorted(urls.keys()))
 
 
-def object_urls(obj):
-  """Returns an object's unique URLs, preserving order.
-  """
-  if isinstance(obj, str):
-    return obj
-  return uniquify(util.trim_nulls(
-    [obj.get('url')] + [u.get('value') for u in obj.get('urls', [])]))
-
-
 def author_display_name(hcard):
   """Returns a human-readable string display name for an h-card object."""
   name = None
@@ -1272,48 +1276,3 @@ def size_to_bytes(size):
     return humanfriendly.parse_size(size)
   except humanfriendly.InvalidSize:
     logger.debug(f"Couldn't parse size {size!r}")
-
-
-def prefix_image_urls(activity, prefix):
-  """Adds a prefix to all image url fields, eg to inject a caching proxy.
-
-  Skips any image url fields that already start with the prefix. URLs are *not*
-  URL-encode before adding the prefix. (This is currently used with
-  https://cloudimage.io/ , which doesn't URL-decode.)
-
-  Args:
-    activity: dict, AS1 activity. Modified in place.
-    prefix: string
-  """
-  a = activity
-  for elem in ([a, a.get('object'), a.get('author'), a.get('actor')] +
-               a.get('replies', {}).get('items', []) +
-               a.get('attachments', []) +
-               a.get('tags', [])):
-    if elem:
-      for img in util.get_list(elem, 'image'):
-        url = img.get('url')
-        if url and not url.startswith(prefix):
-          # Note that url isn't URL-encoded here, that's intentional.
-          # cloudimage.io doesn't decode it.
-          img['url'] = prefix + url
-      if elem is not a:
-        prefix_image_urls(elem, prefix)
-
-
-# TODO: unify
-def prefix_video_urls(activity, prefix):
-  a = activity
-  for elem in ([a, a.get('object'), a.get('author'), a.get('actor')] +
-               a.get('replies', {}).get('items', []) +
-               a.get('attachments', []) +
-               a.get('tags', [])):
-    if elem:
-      for stream in util.get_list(elem, 'stream'):
-        url = stream.get('url')
-        if url and not url.startswith(prefix):
-          # Note that url isn't URL-encoded here, that's intentional.
-          # The caching-proxy Cloudflare worker doesn't decode it.
-          stream['url'] = prefix + url
-      if elem is not a:
-        prefix_video_urls(elem, prefix)
