@@ -9,6 +9,8 @@ import urllib
 
 from . import mastodon
 
+logger = logging.getLogger(__name__)
+
 
 class Pixelfed(mastodon.Mastodon):
   """Pixelfed source class."""
@@ -25,11 +27,10 @@ class Pixelfed(mastodon.Mastodon):
 
   def status_url(self, username, id):
     """Returns the local instance URL for a status with a given id."""
-    return urllib.parse.urljoin(self.instance, '/p/%s/%s' % (
-      urllib.parse.quote(username), id))
+    return urllib.parse.urljoin(self.instance, f'/p/{urllib.parse.quote(username)}/{id}')
 
   def get_activities_response(self, *args, **kwargs):
     if kwargs.get('fetch_mentions'):
-      logging.info("Ignoring fetch_mentions=True since Pixelfed doesn't yet support notifications")
+      logger.info("Ignoring fetch_mentions=True since Pixelfed doesn't yet support notifications")
       kwargs['fetch_mentions'] = False
     return super().get_activities_response(*args, **kwargs)
