@@ -19,103 +19,6 @@ from .. import source
 def tag_uri(name):
   return util.tag_uri('flickr.com', name)
 
-# format for a list of photos like the response to flickr.photos.getRecent
-PHOTOS = {
-  'photos': {
-    'page': 1,
-    'pages': 30,
-    'perpage': 50,
-    'total': '1461',
-    'photo': [
-      {
-        'id': '8998787742',
-        'owner': '39216764@N00',
-        'secret': '89e6e03647',
-        'server': '7459',
-        'farm': 8,
-        'title': 'Percheron Thunder',
-        'ispublic': 1,
-        'isfriend': 0,
-        'isfamily': 0,
-        'dateupload': '1370799634',
-        'datetaken': '2013-06-08 03:20:48',
-        'datetakengranularity': 0,
-        'datetakenunknown': 0,
-        'views': '196',
-        'tags': 'oregon sistersrodeo',
-        'machine_tags': '',
-        'latitude': 0,
-        'longitude': 0,
-        'accuracy': 0,
-        'context': 0,
-        'media': 'photo',
-        'media_status': 'ready'
-      },
-      {
-        'id': '5227327325',
-        'owner': '39216764@N00',
-        'secret': 'ec6180fb32',
-        'server': '4104',
-        'farm': 5,
-        'title': 'Emo dog & pun',
-        'ispublic': 1,
-        'isfriend': 0,
-        'isfamily': 0,
-        'dateupload': '1291338979',
-        'datetaken': '2010-11-27 12:54:33',
-        'datetakengranularity': 0,
-        'datetakenunknown': 0,
-        'views': '176',
-        'tags': 'idyllwild emodog',
-        'machine_tags': '',
-        'latitude': 33.746663,
-        'longitude': -116.711668,
-        'accuracy': 15,
-        'context': 0,
-        'place_id': 'vaJN5r5TVrjDquPG',
-        'woeid': '2426547',
-        'geo_is_family': 0,
-        'geo_is_friend': 0,
-        'geo_is_contact': 0,
-        'geo_is_public': 1,
-        'media': 'photo',
-        'media_status': 'ready'
-      },
-      {
-        'id': '5227922370',
-        'owner': '39216764@N00',
-        'secret': '5f19cb9767',
-        'server': '5246',
-        'farm': 6,
-        'title': 'Candy canes',
-        'ispublic': 1,
-        'isfriend': 0,
-        'isfamily': 0,
-        'dateupload': '1291338921',
-        'datetaken': '2010-11-26 17:50:30',
-        'datetakengranularity': 0,
-        'datetakenunknown': 0,
-        'views': '102',
-        'tags': 'idyllwild',
-        'machine_tags': '',
-        'latitude': 33.746288,
-        'longitude': -116.712441,
-        'accuracy': 16,
-        'context': 0,
-        'place_id': 'vaJN5r5TVrjDquPG',
-        'woeid': '2426547',
-        'geo_is_family': 0,
-        'geo_is_friend': 0,
-        'geo_is_contact': 0,
-        'geo_is_public': 1,
-        'media': 'photo',
-        'media_status': 'ready'
-      }
-    ]
-  },
-  'stat': 'ok'
-}
-
 # format for a single photos like the response to flickr.photos.getInfo
 PHOTO_INFO = {
   'photo': {
@@ -1108,22 +1011,29 @@ class FlickrTest(testutil.TestCase):
     self.assertEqual('123456', reply_content['id'])
     self.assertEqual('https://www.flickr.com/comment/123456', reply_content['url'])
 
-  def test_create_favorite(self):
+  def test_create_favorite_with_like_verb(self):
+    self._test_create_favorite('like')
+
+  def test_create_favorite_with_favorite_verb(self):
+    self._test_create_favorite('favorite')
+
+  def _test_create_favorite(self, verb):
     """Favoriting a photo generates a URL using a fake fragment id
     """
     self.expect_call_api_method(
       'flickr.favorites.add', {'photo_id': '21904325000'},
       json_dumps({'stat': 'ok'}))
-
     self.expect_call_api_method(
       'flickr.people.getLimits', {},
       json_dumps({'person': {'nsid': '39216764@N00'}}))
-
     self.mox.ReplayAll()
+
+    obj = copy.deepcopy(LIKE_OBJ)
+    obj['verb'] = verb
     self.assertEqual({
       'type': 'like',
       'url': 'https://www.flickr.com/photos/marietta_wood_works/21904325000/in/contacts/#favorited-by-39216764@N00',
-    }, self.flickr.create(LIKE_OBJ).content)
+    }, self.flickr.create(obj).content)
 
   def test_photo_to_activity_uses_path_alias_if_username_has_spaces(self):
     photo = PHOTO_INFO['photo']
