@@ -316,7 +316,7 @@ ISSUE_OBJ_WITH_REACTIONS.update({
   'tags': ISSUE_OBJ['tags'] + [REACTION_OBJ, REACTION_OBJ],
   'to': [{'objectType': 'group', 'alias': '@private'}],
 })
-STAR_OBJ = {
+LIKE_OBJ = {
   'objectType': 'activity',
   'verb': 'like',
   'object': {'url': 'https://github.com/foo/bar'},
@@ -353,7 +353,6 @@ NOTIFICATION_ISSUE_REST.update({
 })
 EXPECTED_HEADERS = {
   'Authorization': 'token a-towkin',
-  'Accept': 'application/vnd.github.squirrel-girl-preview+json',
 }
 
 
@@ -498,8 +497,8 @@ class GitHubTest(testutil.TestCase):
     self.expect_rest(REST_NOTIFICATIONS,
                      [NOTIFICATION_PULL_REST, NOTIFICATION_ISSUE_REST])
     self.expect_rest(NOTIFICATION_PULL_REST['subject']['url'], PULL_REST)
-    self.expect_rest(REST_REACTIONS % ('foo', 'bar', 444), [])
     self.expect_rest(NOTIFICATION_ISSUE_REST['subject']['url'], ISSUE_REST)
+    self.expect_rest(REST_REACTIONS % ('foo', 'bar', 444), [])
     self.expect_rest(REST_REACTIONS % ('foo', 'bar', 333),
                      [REACTION_REST, REACTION_REST])
     self.mox.ReplayAll()
@@ -519,6 +518,19 @@ class GitHubTest(testutil.TestCase):
     self.mox.ReplayAll()
     self.assert_equals([ISSUE_OBJ], self.gh.get_activities(activity_id='foo:bar:123'))
 
+  def test_get_activities_activity_id_fetch_replies_likes(self):
+    self.expect_rest(REST_ISSUE % ('foo', 'bar', 333), ISSUE_REST)
+    self.expect_rest(ISSUE_REST['comments_url'], [COMMENT_REST, COMMENT_REST])
+    self.expect_rest(REST_REACTIONS % ('foo', 'bar', 333), [REACTION_REST, REACTION_REST])
+    self.mox.ReplayAll()
+
+    expected = copy.deepcopy(ISSUE_OBJ_WITH_REPLIES)
+    expected['tags'].extend([REACTION_OBJ, REACTION_OBJ])
+    del expected['to']
+
+    self.assert_equals([expected], self.gh.get_activities(
+      activity_id='foo:bar:333', fetch_replies=True, fetch_likes=True))
+
   def test_get_activities_etag_and_since(self):
     self.expect_rest(REST_NOTIFICATIONS, [NOTIFICATION_ISSUE_REST],
                      headers={'If-Modified-Since': 'Thu, 25 Oct 2012 15:16:27 GMT'},
@@ -1005,7 +1017,13 @@ class GitHubTest(testutil.TestCase):
       self.assertIn('Please remove the fragment #bad-456 from your in-reply-to URL.',
                     result.error_plain)
 
-  def test_create_star(self):
+  def test_create_star_with_like_verb(self):
+    self._test_create_star('like')
+
+  def test_create_star_with_favorite_verb(self):
+    self._test_create_star('favorite')
+
+  def _test_create_star(self, verb):
     self.expect_graphql(json={
       'query': github.GRAPHQL_REPO % {
         'owner': 'foo',
@@ -1029,13 +1047,23 @@ class GitHubTest(testutil.TestCase):
     })
     self.mox.ReplayAll()
 
-    result = self.gh.create(STAR_OBJ)
+    obj = copy.deepcopy(LIKE_OBJ)
+    obj['verb'] = verb
+    result = self.gh.create(obj)
     self.assert_equals({
       'url': 'https://github.com/foo/bar/stargazers',
     }, result.content, result)
 
-  def test_preview_star(self):
-    preview = self.gh.preview_create(STAR_OBJ)
+  def test_preview_star_with_like_verb(self):
+    self._test_preview_star('like')
+
+  def test_preview_star_with_favorite_verb(self):
+    self._test_preview_star('favorite')
+
+  def _test_preview_star(self, verb):
+    obj = copy.deepcopy(LIKE_OBJ)
+    obj['verb'] = verb
+    preview = self.gh.preview_create(obj)
     self.assertEqual('<span class="verb">star</span> <a href="https://github.com/foo/bar">foo/bar</a>.', preview.description, preview)
 
   def test_create_reaction_issue(self):
@@ -1144,3 +1172,43 @@ class GitHubTest(testutil.TestCase):
       self.gh._content_for_create({
         'content': 'x <a href="http://foo/bar">http://foo/bar</a> y',
       }))
+
+  def test_create_convert_profile_url_to_mention(self):
+    self.expect_graphql_issue()
+    rendered = self.expect_markdown_render('x @foo y')
+    self.mox.ReplayAll()
+
+    comment = copy.deepcopy(COMMENT_OBJ)
+    comment['content'] = 'x https://github.com/foo y'
+    self.gh.preview_create(comment)
+
+  def test_create_profile_url_in_html_link(self):
+    self.expect_graphql_issue()
+    rendered = self.expect_markdown_render('[text](https://github.com/foo)')
+    self.mox.ReplayAll()
+
+    comment = copy.deepcopy(COMMENT_OBJ)
+    comment['content'] = '<a href="https://github.com/foo">text</a>'
+    self.gh.preview_create(comment)
+
+  def test_create_convert_profile_url_to_mention_unchanged(self):
+    comment = copy.deepcopy(COMMENT_OBJ)
+
+    for input in (
+        'https://github.com.com/foo',
+        'github.com/foo',
+        'https://github.com/foo/bar',
+        'https://github.com/foo?bar=baz',
+        'https://github.com/not@user+name',
+    ):
+      with self.subTest(input):
+        self.tearDown()
+        self.mox.UnsetStubs()
+        self.mox.ResetAll()
+        self.setUp()
+        self.expect_graphql_issue()
+        rendered = self.expect_markdown_render(input)
+
+        self.mox.ReplayAll()
+        comment['content'] = input
+        self.gh.preview_create(comment)
