@@ -415,7 +415,7 @@ ATOM = """\
 
   <activity:object-type>http://activitystrea.ms/schema/1.0/photo</activity:object-type>
 
-  <id>https://www.instagram.com/p/ABC123/</id>
+  <id>tag:instagram.com:123_456</id>
   <title>this picture -&gt; is #abc @foo #xyz</title>
 
   <content type="xhtml">
@@ -706,7 +706,7 @@ HTML_VIEWER_CONFIG = {
     'username': 'snarfed',
   },
 }
-HTML_FEED = {  # eg https://www.instagram.com/ when you're logged in
+HTML_FEED = {  # eg old https://www.instagram.com/ when you're logged in
   'environment_switcher_visible_server_guess': True,
   'config': HTML_VIEWER_CONFIG,
   'display_properties_server_guess': {'pixel_ratio': 2.0, 'viewport_width': 1280},
@@ -734,6 +734,13 @@ HTML_FEED = {  # eg https://www.instagram.com/ when you're logged in
     },
   }}}]},
 }
+HTML_DEFINES = {  # eg new https://www.instagram.com/ when you're logged in
+  'define': [
+    ["IntlCurrentLocale", [], {"code": "en_US"}, 5954],
+    ["CookieDomain", [], {"domain": "instagram.com"}, 6421],
+    ["XIGSharedData", [], {"raw": json_dumps(HTML_FEED), 'native': '...'}, 6186],
+  ],
+}
 
 # Included with window.__additionalDataLoaded('feed_v2', ...)
 # Extracted from https://www.instagram.com/ on 2021-11-24
@@ -881,18 +888,11 @@ HTML_VIDEO_V2_FULL = {
   'like_count': 9,
 
   'usertags': {
-    'in': [
-      {
-        'user': {
-          # 'pk': 654,
-          'username': 'ap',
-          # 'full_name': 'A P',
-          # 'is_private': False,
-          # 'profile_pic_url': 'https://scontent-sjc3-1.cdninstagram.com/v/t51.2885-19/s150x150/654_33_22_n.jpg?...',
-          # 'profile_pic_id': '2629_654',
-        },
+    'in': [{
+      'user': {
+        'username': 'ap',
       },
-    ],
+    }],
   },
 }
 
@@ -964,6 +964,36 @@ HTML_PROFILE = {  # eg https://www.instagram.com/snarfed
 HTML_PROFILE_PRIVATE = copy.deepcopy(HTML_PROFILE)
 HTML_PROFILE_PRIVATE['entry_data']['ProfilePage'][0]['graphql']['user']['is_private'] = True
 
+# eg https://i.instagram.com/api/v1/users/web_profimle_info/?username=snarfed
+# HTTP fetch needs Cookie and X-IG-App-ID headers
+# data is similar to HTML_PRELOAD_DATA
+HTML_PROFILE_JSON = {
+  'data': {
+    'user': {
+      'edge_owner_to_timeline_media': {
+        'count': 159,
+        'edges': [
+          {'node': HTML_PHOTO},
+          {'node': HTML_VIDEO},
+        ],
+      },
+      'id': '420973239',
+      'fbid': '17841401357176577',
+      'username': 'snarfed',
+      'full_name': 'Ryan B',
+      'external_url': 'https://snarfed.org',
+      'biography': 'something or other',
+      'is_private': False,
+      'is_verified': False,
+      'profile_pic_url': 'https:\/\/scontent-sjc2-1.cdninstagram.com\/hphotos-xfa1\/t51.2885-19\/11373714_959073410822287_2004790583_a.jpg',
+      'profile_pic_url_hd': 'https:\/\/scontent-sjc2-1.cdninstagram.com\/hphotos-xfa1\/t51.2885-19\/11373714_959073410822287_2004790583_hd.jpg',
+      'followed_by_viewer': False,
+      'follows_viewer': False,
+      'edge_followed_by': {'count': 168},
+      'edge_follow': {'count': 111},
+    },
+  },
+}
 HTML_PHOTO_PAGE = {  # eg https://www.instagram.com/p/ABC123/
   'config': {
     'csrf_token': 'xyz',
@@ -1007,6 +1037,7 @@ HTML_USELESS_FEED = {
   },
 }
 # eg https://www.instagram.com/graphql/query/?query_hash=d6f4...&variables={}
+# data is similar to HTML_PROFILE_JSON
 HTML_PRELOAD_DATA = {
   'data': {
     'user': {
@@ -1022,7 +1053,10 @@ HTML_PRELOAD_DATA = {
       },
       'id': '420973239',
       'username': 'snarfed',
-      'profile_pic_url': 'https://scontent-ort2-1.cdninstagram.com/vp/a14b81a5e52a25f456b0e4253ee2949c/5B2DF84C/t51.2885-19/11373714_959073410822287_2004790583_a.jpg',
+      'full_name': 'Ryan B',
+      'profile_pic_url': 'https://scontent-sjc2-1.cdninstagram.com/hphotos-xfa1/t51.2885-19/11373714_959073410822287_2004790583_a.jpg',
+      'external_url': 'https://snarfed.org',
+      'biography': 'something or other',
     }
   },
   'status': 'ok',
@@ -1040,6 +1074,7 @@ HTML_HEADER_TEMPLATE = """
 HTML_HEADER = HTML_HEADER_TEMPLATE % ('', 'window._sharedData = ')
 HTML_HEADER_2 = HTML_HEADER_TEMPLATE % ('', "window.__additionalDataLoaded('feed', ")
 HTML_HEADER_3 = HTML_HEADER_TEMPLATE % ('', "window.__additionalDataLoaded('/p/B3Q5Fa8Ja4D/', ")
+
 HTML_PRELOAD_URL = '/graphql/query/?query_hash=cba321&variables={}'
 HTML_HEADER_PRELOAD = HTML_HEADER_TEMPLATE % (
   f'<link rel="preload" href="{HTML_PRELOAD_URL}" as="fetch" type="application/json" crossorigin />',
@@ -1208,6 +1243,23 @@ HTML_MULTI_PHOTO_ACTIVITY['object']['attachments'] = [{
   }],
 }]
 
+HTML_V2_SUGGESTED_USERS = {
+  'suggested_users': {
+    'type': 2,
+    'suggestions': [
+      # ...
+    ],
+    'landing_site_type': 'suggested_user',
+    'title': 'Suggested for You',
+    'view_all_text': 'See All',
+    'landing_site_title': 'Discover People',
+    'cards_size': 'large',
+    'id': '60155560',
+    'tracking_token': '...',
+    # ...
+  },
+}
+
 HTML_ACTIVITIES = [HTML_PHOTO_ACTIVITY, HTML_VIDEO_ACTIVITY]
 HTML_ACTIVITIES_FULL = [HTML_PHOTO_ACTIVITY_FULL, HTML_VIDEO_ACTIVITY_FULL]
 HTML_ACTIVITIES_FULL_LIKES = [HTML_PHOTO_ACTIVITY_LIKES, HTML_VIDEO_ACTIVITY_FULL]
@@ -1216,6 +1268,14 @@ HTML_FEED_COMPLETE = HTML_HEADER + json_dumps(HTML_FEED) + HTML_FOOTER
 
 HTML_FEED_COMPLETE_2 = HTML_HEADER_2 + json_dumps(HTML_PRELOAD_DATA['data']) + ')' + HTML_FOOTER
 
+HTML_FEED_COMPLETE_4 = """\
+<!DOCTYPE html><html class="..."><script nonce="..."></script>
+...
+<body>
+...
+<script>requireLazy(["JSScheduler","ServerJS","ScheduledApplyEach"],function(JSScheduler,ServerJS,ScheduledApplyEach){qpl_inl("...","tierOneBeforeScheduler");JSScheduler.runWithPriority(3,function(){qpl_inl("...","tierOneInsideScheduler");(new ServerJS()).handleWithCustomApplyEach(ScheduledApplyEach,\
+""" + json_dumps(HTML_DEFINES) + ');});})' + HTML_FOOTER
+
 HTML_FEED_COMPLETE_V2 = HTML_HEADER + json_dumps(HTML_FEED_V2) + HTML_FOOTER
 HTML_PHOTO_ACTIVITY_V2_FULL = copy.deepcopy(HTML_PHOTO_ACTIVITY_FULL)
 HTML_PHOTO_ACTIVITY_V2_FULL['object']['location'].update({
@@ -1407,7 +1467,7 @@ class InstagramTest(testutil.TestCase):
 
     self.mox.ReplayAll()
 
-    cache = util.CacheDict()
+    cache = {}
     for _ in range(3):
       self.instagram.get_activities(user_id='x', group_id=source.SELF,
                                     fetch_likes=True, fetch_replies=True,
@@ -1803,9 +1863,12 @@ class InstagramTest(testutil.TestCase):
     self.assert_equals(HTML_ACTIVITIES_FULL, activities)
     self.assert_equals(HTML_VIEWER, viewer)
 
-    activities, viewer = self.instagram.scraped_to_activities(HTML_FEED_COMPLETE_2)
+    activities, _ = self.instagram.scraped_to_activities(HTML_FEED_COMPLETE_2)
     self.assert_equals(HTML_ACTIVITIES_FULL, activities)
 
+    _, viewer = self.instagram.scraped_to_activities(HTML_FEED_COMPLETE_4)
+    self.assert_equals(HTML_VIEWER, viewer)
+
   def test_scraped_to_activities_feed_v2(self):
     activities, viewer = self.instagram.scraped_to_activities(HTML_FEED_COMPLETE_V2)
     self.assert_equals(HTML_ACTIVITIES_FULL_V2, activities)
@@ -1849,6 +1912,23 @@ class InstagramTest(testutil.TestCase):
     self.assertEqual(expected, activities[0]['actor'])
     self.assertEqual(expected, activities[0]['object']['author'])
 
+  def test_scraped_json_to_activities_profile(self):
+    activities, actor = self.instagram.scraped_json_to_activities(HTML_PROFILE_JSON)
+    self.assert_equals(HTML_ACTIVITIES, activities)
+    self.assert_equals(HTML_VIEWER_PUBLIC, actor)
+
+  def test_scraped_to_activities_json_input(self):
+    activities, actor = self.instagram.scraped_to_activities(
+      json_dumps(HTML_PROFILE_JSON))
+    self.assert_equals(HTML_ACTIVITIES, activities)
+    self.assert_equals(HTML_VIEWER_PUBLIC, actor)
+
+  def test_scraped_json_to_activities_suggested_users(self):
+    activities, actor = self.instagram.scraped_json_to_activities(
+      {'feed_items': [HTML_V2_SUGGESTED_USERS]})
+    self.assert_equals([], activities)
+    self.assert_equals(None, actor)
+
   def test_scraped_to_activities_photo_no_fetch_extras(self):
     activities, viewer = self.instagram.scraped_to_activities(
       HTML_PHOTO_COMPLETE, fetch_extras=False)
@@ -1856,8 +1936,10 @@ class InstagramTest(testutil.TestCase):
     self.assertIsNone(viewer)
 
   def test_scraped_to_activities_photo_fetch_extras(self):
+    self.instagram.cookie = 'kuky'
     self.expect_requests_get(
-      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE, headers={})
+      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE,
+      headers=mox.IgnoreArg())
     self.mox.ReplayAll()
 
     activities, viewer = self.instagram.scraped_to_activities(
@@ -1879,8 +1961,10 @@ class InstagramTest(testutil.TestCase):
                        self.instagram.scraped_to_activity(html))
 
   def test_scraped_to_activity_photo_fetch_extras(self):
+    self.instagram.cookie = 'kuky'
     self.expect_requests_get(
-      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE, headers={})
+      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE,
+      headers=mox.IgnoreArg())
     self.mox.ReplayAll()
 
     activity, actor = self.instagram.scraped_to_activity(
@@ -1890,8 +1974,10 @@ class InstagramTest(testutil.TestCase):
 
   def test_scraped_to_activities_photo_edge_media_to_parent_comment(self):
     """https://github.com/snarfed/granary/issues/164"""
+    self.instagram.cookie = 'kuky'
     self.expect_requests_get(
-      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE, headers={})
+      instagram.HTML_LIKES_URL % 'ABC123', HTML_PHOTO_LIKES_RESPONSE,
+      headers=mox.IgnoreArg())
     self.mox.ReplayAll()
 
     page = copy.deepcopy(HTML_PHOTO_PAGE)
@@ -2053,7 +2139,7 @@ class InstagramTest(testutil.TestCase):
 
     self.assert_equals([HTML_PHOTO_ACTIVITY_FULL, HTML_VIDEO_ACTIVITY_FULL],
                        activities)
-    self.assertIsNone(viewer)
+    self.assert_equals(HTML_VIEWER, viewer)
 
   def test_scraped_to_activities_preload_fetch_bad_json(self):
     """https://console.cloud.google.com/errors/CP_w8ai-7JLfvAE"""
