@@ -4,6 +4,7 @@ import copy
 
 from oauth_dropins.webutil import testutil, util
 from oauth_dropins.webutil.util import json_dumps, json_loads
+from requests import HTTPError
 
 from .. import mastodon
 from .. import source
@@ -324,6 +325,7 @@ class MastodonTest(testutil.TestCase):
     kwargs.setdefault('headers', {}).update({
       'Authorization': 'Bearer towkin',
     })
+    kwargs.setdefault('content_type', 'application/json; charset=utf-8')
     return fn(INSTANCE + path, response=response, **kwargs)
 
   def test_constructor_look_up_user_id(self):
@@ -366,7 +368,7 @@ class MastodonTest(testutil.TestCase):
     with_replies['object']['replies'] = {
         'items': [REPLY_ACTIVITY, REPLY_ACTIVITY],
     }
-    cache = util.CacheDict()
+    cache = {}
     self.assert_equals([with_replies], self.mastodon.get_activities(
       fetch_replies=True, cache=cache))
     self.assert_equals(1, cache['AMRE 123'])
@@ -378,7 +380,7 @@ class MastodonTest(testutil.TestCase):
 
     with_likes = copy.deepcopy(ACTIVITY)
     with_likes['object']['tags'].extend([LIKE, LIKE])
-    cache = util.CacheDict()
+    cache = {}
     self.assert_equals([with_likes], self.mastodon.get_activities(
       fetch_likes=True, cache=cache))
     self.assert_equals(2, cache['AMF 123'])
@@ -390,7 +392,7 @@ class MastodonTest(testutil.TestCase):
 
     with_shares = copy.deepcopy(ACTIVITY)
     with_shares['object']['tags'].extend([SHARE, SHARE_BY_REMOTE])
-    cache = util.CacheDict()
+    cache = {}
     self.assert_equals([with_shares], self.mastodon.get_activities(
       fetch_shares=True, cache=cache))
     self.assert_equals(3, cache['AMRB 123'])
@@ -508,11 +510,20 @@ class MastodonTest(testutil.TestCase):
       self.expect_get(API_TIMELINE, params={}, response=statuses)
 
     self.mox.ReplayAll()
-    cache = util.CacheDict()
+    cache = {}
     for _ in range(4):
       self.mastodon.get_activities(fetch_replies=True, fetch_shares=True,
                                    fetch_likes=True, cache=cache)
 
+  def test_get_activities_returns_non_json(self):
+    self.expect_get(API_TIMELINE, params={}, response='<html>',
+                    content_type='text/html')
+    self.mox.ReplayAll()
+
+    with self.assertRaises(HTTPError) as e:
+      self.mastodon.get_activities()
+    self.assert_equals(502, e.exception.response.status_code)
+
   def test_get_actor(self):
     self.expect_get(API_ACCOUNT % 1, ACCOUNT)
     self.mox.ReplayAll()
@@ -695,6 +706,7 @@ class MastodonTest(testutil.TestCase):
       'content': 'foo ☕ bar',
       'inReplyTo': [{'url': url}],
     })
+    self.assert_equals(STATUS, got.content, got)
 
   def test_preview_reply_remote(self):
     url = STATUS_REMOTE['url']
@@ -711,16 +723,44 @@ class MastodonTest(testutil.TestCase):
       preview.description)
     self.assert_equals('foo ☕ bar', preview.content)
 
-  def test_create_favorite(self):
+  def test_create_non_mastodon_reply(self):
+    self.expect_get(API_SEARCH, params={'q': 'http://not/mastodon', 'resolve': True},
+                    response={})
+    self.expect_post(API_STATUSES, json={'status': 'foo ☕ bar'}, response=STATUS)
+    self.mox.ReplayAll()
+
+    got = self.mastodon.create({
+      'content': 'foo ☕ bar',
+      'inReplyTo': [{'url': 'http://not/mastodon'}],
+    })
+    self.assert_equals(STATUS, got.content, got)
+
+  def test_create_favorite_with_like_verb(self):
+    self._test_create_favorite('like')
+
+  def test_create_favorite_with_favorite_verb(self):
+    self._test_create_favorite('favorite')
+
+  def _test_create_favorite(self, verb):
     self.expect_post(API_FAVORITE % '123', STATUS)
     self.mox.ReplayAll()
 
-    got = self.mastodon.create(LIKE).content
+    obj = copy.deepcopy(LIKE)
+    obj['verb'] = verb
+    got = self.mastodon.create(obj).content
     self.assert_equals('like', got['type'])
     self.assert_equals('http://foo.com/@snarfed/123#favorited-by-23507', got['url'])
 
-  def test_preview_favorite(self):
-    preview = self.mastodon.preview_create(LIKE)
+  def test_preview_favorite_with_like_verb(self):
+    self._test_preview_favorite('like')
+
+  def test_preview_favorite_with_favorite_verb(self):
+    self._test_preview_favorite('favorite')
+
+  def _test_preview_favorite(self, verb):
+    obj = copy.deepcopy(LIKE)
+    obj['verb'] = verb
+    preview = self.mastodon.preview_create(obj)
     self.assertIn('<span class="verb">favorite</span> <a href="http://foo.com/@snarfed/123">this toot</a>: ', preview.description)
 
   def test_create_favorite_remote(self):
