@@ -588,6 +588,17 @@ Shared <a href="#">a post</a> by foo
       'content': None,
     }))
 
+  def test_object_to_json_html_entities(self):
+    self.assert_equals({
+      'type': ['h-entry'],
+      'properties': {
+        'content': ['foo " bar > > >']
+      },
+    }, microformats2.object_to_json({
+      'objectType': 'activity',
+      'content': 'foo &quot; bar &gt; &#62; &#x3e;',
+    }))
+
   def test_get_string_urls(self):
     for expected, objs in (
         ([], []),
@@ -803,23 +814,6 @@ Shared <a href="#">a post</a> by foo
 </body>
 """, url='http://123')))
 
-  def test_object_urls(self):
-    for expected, actor in (
-        ([], {}),
-        ([], {'displayName': 'foo'}),
-        ([], {'url': None, 'urls': []}),
-        (['http://foo'], {'url': 'http://foo'}),
-        (['http://foo'], {'urls': [{'value': 'http://foo'}]}),
-        (['http://foo', 'https://bar', 'http://baz'], {
-          'url': 'http://foo',
-          'urls': [{'value': 'https://bar'},
-                   {'value': 'http://foo'},
-                   {'value': 'http://baz'},
-          ],
-        }),
-    ):
-      self.assertEqual(expected, microformats2.object_urls(actor))
-
   def test_hcard_to_html_no_properties(self):
     self.assertEqual('', microformats2.hcard_to_html({}))
     self.assertEqual('', microformats2.hcard_to_html({'properties': {}}))
@@ -865,7 +859,7 @@ Shared <a href="#">a post</a> by   <span class="h-card">
 <!DOCTYPE html>
 <html>
 <head><meta charset="utf-8"></head>
-<body>
+<body class="">
 <article class="h-entry">
   <span class="p-uid">http://localhost:3000/users/ryan#likes/7</span>
   <span class="p-author h-card">
@@ -937,28 +931,3 @@ Shared <a href="#">a post</a> by   <span class="h-card">
         ('5 MB', 5000000),
     ):
       self.assertEqual(expected, microformats2.size_to_bytes(input), input)
-
-  def test_prefix_image_urls(self):
-    IMG = lambda: copy.deepcopy({'image': {'url': 'xyz'}})
-    PRE = lambda: copy.deepcopy({'image': {'url': 'abcxyz'}})
-
-    activity = {
-      'actor': IMG(),
-      'object': {
-        'author': IMG(),
-        'replies': {'items': [IMG(), IMG(), PRE()]},
-        'attachments': [IMG(), {'author': IMG()}],
-        'tags': [IMG(), {'actor': IMG()}],
-      },
-    }
-    microformats2.prefix_image_urls(activity, 'abc')
-
-    self.assert_equals({
-      'actor': PRE(),
-      'object': {
-        'author': PRE(),
-        'replies': {'items': [PRE(), PRE(), PRE()]},
-        'attachments': [PRE(), {'author': PRE()}],
-        'tags': [PRE(), {'actor': PRE()}],
-      },
-    }, activity)
