@@ -8,6 +8,10 @@ http://gdata-python-client.googlecode.com/hg/pydocs/gdata.blogger.data.html
 
 Uses requests-oauthlib to auth via Google Sign-In's OAuth 2:
 https://requests-oauthlib.readthedocs.io/
+
+Known issues:
+* If the user approves the OAuth prompt but has no Blogger blogs, we redirect to
+  the callback with declined=True, which is wrong.
 """
 import logging
 import re
@@ -22,6 +26,8 @@ from .webutil import flask_util
 from .webutil import util
 from .webutil.util import json_dumps, json_loads
 
+logger = logging.getLogger(__name__)
+
 AUTH_CODE_URL = 'https://accounts.google.com/o/oauth2/v2/auth'
 ACCESS_TOKEN_URL = 'https://www.googleapis.com/oauth2/v4/token'
 
@@ -73,7 +79,7 @@ class BloggerV2Auth(models.BaseAuth):
     :meth:`gdata.client.GDClient.request`. Other similar classes include
     :class:`gdata.gauth.ClientLoginToken` and :class:`gdata.gauth.AuthSubToken`.
     """
-    http_request.headers['Authorization'] = 'Bearer %s' % self.access_token()
+    http_request.headers['Authorization'] = f'Bearer {self.access_token()}'
 
 
 class Scopes(object):
@@ -116,8 +122,8 @@ class Callback(Scopes, views.Callback):
     error = request.values.get('error')
     desc = request.values.get('error_description')
     if error:
-      msg = 'Error: %s: %s' % (error, desc)
-      logging.info(msg)
+      msg = f'Error: {error}: {desc}'
+      logger.info(msg)
       if error == 'access_denied':
         return self.finish(None, state=state)
       else:
@@ -136,8 +142,10 @@ class Callback(Scopes, views.Callback):
     except BaseException as e:
       # this api call often returns 401 Unauthorized for users who aren't
       # signed up for blogger and/or don't have any blogs.
+      # TODO: propagate this info up. Right now this makes self.finish() add a
+      # declined=True query param to the callback, which is wrong.
       util.interpret_http_exception(e)
-      return
+      return self.finish(None, state=state)
 
     for id in ([a.uri.text for a in blogs.author if a.uri] +
                [l.href for l in blogs.link if l]):
@@ -147,7 +155,7 @@ class Callback(Scopes, views.Callback):
       if match:
         id = match.group(1)
       else:
-        logging.warning("Couldn't parse %s , using entire value as id", id)
+        logger.warning(f"Couldn't parse {id} , using entire value as id")
       break
 
     blog_ids = []
