@@ -14,6 +14,8 @@ from . import views, models
 from .webutil import flask_util, util
 from .webutil.util import json_dumps, json_loads
 
+logger = logging.getLogger(__name__)
+
 DROPBOX_APP_KEY = util.read('dropbox_app_key')
 DROPBOX_APP_SECRET = util.read('dropbox_app_secret')
 GET_AUTH_CODE_URL = '&'.join((
@@ -60,7 +62,7 @@ class DropboxAuth(models.BaseAuth):
   def urlopen(self, url, **kwargs):
     """Wraps urlopen() and adds OAuth credentials to the request.
     """
-    headers = {'Authorization': 'Bearer %s' % self.access_token_str}
+    headers = {'Authorization': f'Bearer {self.access_token_str}'}
     try:
       return util.urlopen(urllib.request.Request(url, headers=headers), **kwargs)
     except BaseException as e:
@@ -89,7 +91,7 @@ class Start(views.Start):
     return GET_AUTH_CODE_URL % {
       'client_id': DROPBOX_APP_KEY,
       'redirect_uri': urllib.parse.quote_plus(self.to_url(state=state)),
-      'state': '%s|%s' % (state, csrf_key.id()),
+      'state': f'{state}|{csrf_key.id()}',
     }
 
   @classmethod
@@ -110,7 +112,7 @@ class Callback(views.Callback):
 
     if error or error_reason:
       if error == 'access_denied':
-        logging.info('User declined: %s', error_reason)
+        logger.info(f'User declined: {error_reason}')
         return self.finish(None, state=state)
       else:
         flask_util.error(' '.join((error, error_reason)))
@@ -119,11 +121,11 @@ class Callback(views.Callback):
     try:
       csrf_id = int(urllib.parse.unquote_plus(state).split('|')[-1])
     except (ValueError, TypeError):
-      flask_util.error('Invalid state value %r' % state)
+      flask_util.error(f'Invalid state value {state!r}')
 
     csrf = DropboxCsrf.get_by_id(csrf_id)
     if not csrf:
-      flask_util.error('No CSRF token for id %s' % csrf_id)
+      flask_util.error(f'No CSRF token for id {csrf_id}')
 
     # request an access token
     data = {
@@ -141,10 +143,10 @@ class Callback(views.Callback):
     try:
       data = json_loads(resp)
     except (ValueError, TypeError):
-      logging.error('Bad response:\n%s', resp, exc_info=True)
+      logger.error(f'Bad response:\n{resp}', exc_info=True)
       flask_util.error('Bad Dropbox response to access token request')
 
-    logging.info('Storing new Dropbox account: %s', data['uid'])
+    logger.info(f"Storing new Dropbox account: {data['uid']}")
     auth = DropboxAuth(id=data['uid'], access_token_str=data['access_token'])
     auth.put()
     return self.finish(auth, state=csrf.state)
