@@ -23,13 +23,11 @@ from .models import BaseAuth
 from .webutil import flask_util, util
 from .webutil.util import json_dumps, json_loads
 
+logger = logging.getLogger(__name__)
+
 MEDIUM_CLIENT_ID = util.read('medium_client_id')
 MEDIUM_CLIENT_SECRET = util.read('medium_client_secret')
 
-# medium is behind cloudflare, which often blocks requests's user agent, so set
-# our own.
-USER_AGENT = 'oauth-dropins (https://oauth-dropins.appspot.com/)'
-
 # URL templates. Can't (easily) use urlencode() because I want to keep the
 # %(...)s placeholders as is and fill them in later in code.
 GET_AUTH_CODE_URL = '&'.join((
@@ -89,7 +87,6 @@ class MediumAuth(BaseAuth):
     """
     headers = kwargs.setdefault('headers', {})
     headers['Authorization'] = 'Bearer ' + self.access_token_str
-    headers.setdefault('User-Agent', USER_AGENT)
 
     resp = util.requests_get(*args, **kwargs)
     try:
@@ -128,7 +125,7 @@ class Callback(views.Callback):
     error = request.values.get('error')
     if error:
       if error == 'access_denied':
-        logging.info('User declined')
+        logger.info('User declined')
         return self.finish(None, state=request.values.get('state'))
       else:
         flask_util.error(error)
@@ -144,20 +141,19 @@ class Callback(views.Callback):
       'redirect_uri': request.base_url,
       'grant_type': 'authorization_code',
     }
-    resp = util.requests_post(GET_ACCESS_TOKEN_URL, data=data,
-                              headers={'User-Agent': USER_AGENT})
+    resp = util.requests_post(GET_ACCESS_TOKEN_URL, data=data)
     resp.raise_for_status()
-    logging.debug('Access token response: %s', resp.text)
+    logger.debug(f'Access token response: {resp.text}')
 
     try:
       resp = json_loads(resp.text)
     except:
-      logging.error('Could not decode JSON', exc_info=True)
+      logger.error('Could not decode JSON', exc_info=True)
       raise
 
     errors = resp.get('errors') or resp.get('error')
     if errors:
-      logging.info('Errors: %s', errors)
+      logger.info(f'Errors: {errors}')
       flask_util.error(errors[0].get('message'))
 
     # TODO: handle refresh token
