@@ -14,6 +14,8 @@ from .models import BaseAuth
 from .webutil import appengine_info, flask_util, util
 from .webutil.util import json_loads
 
+logger = logging.getLogger(__name__)
+
 if appengine_info.DEBUG:
     MEETUP_CLIENT_ID = util.read('meetup_client_id_local')
     MEETUP_CLIENT_SECRET = util.read('meetup_client_secret_local')
@@ -36,7 +38,7 @@ GET_USER_INFO_URL = 'https://api.meetup.com/members/self/'
 def urlopen_bearer_token(url, access_token, data=None, **kwargs):
     """Wraps urlopen() and adds OAuth credentials to the request.
     """
-    headers = {'Authorization': 'Bearer %s' % access_token}
+    headers = {'Authorization': f'Bearer {access_token}'}
     try:
         return util.urlopen(urllib.request.Request(url, headers=headers, data=data), **kwargs)
     except BaseException as e:
@@ -51,7 +53,7 @@ class MeetupAuth(BaseAuth):
     OAuth-signed requests to Meetup's HTTP-based APIs. Stores OAuth credentials
     in the datastore. See models.BaseAuth for usage details.
 
-    Implements urlopen() but not http() or api().
+    Implements urlopen() but not api().
     """
     access_token_str = ndb.StringProperty(required=True)
     user_json = ndb.TextProperty(required=True)
@@ -98,7 +100,7 @@ class Start(views.Start):
             'client_id': MEETUP_CLIENT_ID,
             'redirect_uri': urllib.parse.quote_plus(self.to_url()),
             'scope': self.scope,
-            'state': '%s|%s' % (urllib.parse.quote_plus(state), csrf_key.id()),
+            'state': f'{urllib.parse.quote_plus(state or "")}|{csrf_key.id()}',
         }
 
     @classmethod
@@ -115,21 +117,21 @@ class Callback(views.Callback):
         error = request.values.get('error')
         if error:
             if error == 'access_denied':
-                logging.info('User declined')
+                logger.info('User declined')
                 return self.finish(None, state=request.values.get('state'))
             else:
-                flask_util.error('Error: %s: %s' % (error, request.values.get('error_description')))
+                flask_util.error(f"Error: {error}: {request.values.get('error_description')}")
 
         state = request.values['state']
         # lookup the CSRF token
         try:
             csrf_id = int(urllib.parse.unquote_plus(state).split('|')[-1])
         except (ValueError, TypeError):
-            flask_util.error('Invalid state value %r' % state)
+            flask_util.error(f'Invalid state value {state!r}')
 
         csrf = MeetupCsrf.get_by_id(csrf_id)
         if not csrf:
-            flask_util.error('No CSRF token for id %s' % csrf_id)
+            flask_util.error(f'No CSRF token for id {csrf_id}')
 
         # extract auth code and request access token
         auth_code = request.values['code']
@@ -146,27 +148,27 @@ class Callback(views.Callback):
         resp = util.requests_post(GET_ACCESS_TOKEN_URL, data=data)
         resp.raise_for_status()
 
-        logging.debug('Access token response: %s', resp)
+        logger.debug(f'Access token response: {resp}')
 
         try:
             data = json_loads(resp.text)
         except (ValueError, TypeError):
-            logging.error('Bad response:\n%s', resp, exc_info=True)
+            logger.error(f'Bad response:\n{resp}', exc_info=True)
             flask_util.error('Bad Disqus response to access token request')
 
         error = data.get('error')
         if error:
-            msg = 'Error: %s: %s' % (error[0], data.get('error_description'))
+            msg = f"Error: {error[0]}: {data.get('error_description')}"
             flask_util.error(msg)
 
         access_token = data['access_token']
 
         user_json = urlopen_bearer_token(GET_USER_INFO_URL, access_token).read()
         user = json_loads(user_json)
-        logging.debug('User info response: %s', user)
+        logger.debug(f'User info response: {user}')
         user_id = str(user['id'])
 
-        logging.info('Storing new Meetup account for ID: %s', user_id)
+        logger.info(f'Storing new Meetup account for ID: {user_id}')
         auth = MeetupAuth(id=user_id, access_token_str=access_token, user_json=user_json)
         auth.put()
         return self.finish(auth, state=csrf.state)
