@@ -4,7 +4,7 @@ StackDriver Logging API:
 https://cloud.google.com/logging/docs/apis
 """
 import calendar
-import datetime
+from datetime import datetime, timedelta, timezone
 import html
 import logging
 import re
@@ -13,12 +13,14 @@ import urllib.request, urllib.parse, urllib.error
 
 from google.cloud import ndb
 from google.cloud.logging import Client
-import humanize
+from oauth_dropins.webutil.util import json_dumps, json_loads
 
 from .appengine_info import APP_ID
 from . import flask_util, util
 from .flask_util import error
 
+logger = logging.getLogger(__name__)
+
 LEVELS = {
   logging.DEBUG:    'D',
   logging.INFO:     'I',
@@ -27,10 +29,11 @@ LEVELS = {
   logging.CRITICAL: 'F',
 }
 
-CACHE_TIME = datetime.timedelta(days=1)
-MAX_LOG_AGE = datetime.timedelta(days=30)
+CACHE_TIME = timedelta(days=1)
+MAX_LOG_AGE = timedelta(days=30)
 # App Engine's launch, roughly
-MIN_START_TIME = time.mktime(datetime.datetime(2008, 4, 1).timetuple())
+MIN_START_TIME = time.mktime(datetime(2008, 4, 1, tzinfo=timezone.utc).timetuple())
+MAX_START_TIME = time.mktime(datetime(2099, 1, 1, tzinfo=timezone.utc).timetuple())
 
 SANITIZE_RE = re.compile(r"""
   ((?:access|api|oauth)?[ _]?
@@ -48,18 +51,20 @@ def sanitize(msg):
   return SANITIZE_RE.sub(r'\1...', msg)
 
 
-def url(when, key):
+def url(when, key, module=None):
   """Returns the relative URL (no scheme or host) to a log page.
 
   Args:
     when: datetime
-    key: ndb.Key
+    key: ndb.Key or str
+    module: string, optional App Engine module
   """
-  return 'log?start_time=%s&key=%s' % (
-    calendar.timegm(when.utctimetuple()), key.urlsafe().decode())
+  if isinstance(key, ndb.Key):
+    key = key.urlsafe().decode()
+  return f'log?start_time={calendar.timegm(when.utctimetuple())}&key={key}&module={module if module else ""}'
 
 
-def maybe_link(when, key, time_class='dt-updated', link_class=''):
+def maybe_link(when, key, time_class='dt-updated', link_class='', module=None):
   """Returns an HTML snippet with a timestamp and maybe a log page link.
 
   Example:
@@ -78,26 +83,23 @@ def maybe_link(when, key, time_class='dt-updated', link_class=''):
 
   Args:
     when: datetime
-    key: ndb.Key
+    key: ndb.Key or str
     time_class: string, optional class value for the <time> tag
     link_class: string, optional class value for the <a> tag (if generated)
+    module: string, optional App Engine module to search logs of
 
   Returns: string HTML
   """
   # always show time zone. assume naive timestamps are UTC.
   if when.tzinfo is None:
-    when = when.replace(tzinfo=datetime.timezone.utc)
+    when = when.replace(tzinfo=timezone.utc)
 
-  # humanize.naturaltime breaks on timezone-aware datetimes :(
-  # https://github.com/jmoiron/humanize/issues/9#issuecomment-322917865
-  now = datetime.datetime.now(tz=when.tzinfo)
+  now = datetime.now(tz=when.tzinfo)
 
-  time = '<time class="%s" datetime="%s" title="%s %s">%s</time>' % (
-    time_class, when.isoformat(), when.ctime(), when.tzname(),
-    humanize.naturaltime(when, when=now))
+  time = f'<time class="{time_class}" datetime="{when.isoformat()}" title="{when.ctime()} {when.tzname()}">{util.naturaltime(when, when=now)}</time>'
 
   if now > when > now - MAX_LOG_AGE:
-    return '<a class="%s" href="/%s">%s</a>' % (link_class, url(when, key), time)
+    return f'<a class="{link_class}" href="/{url(when, key, module=module)}">{time}</a>'
 
   return time
 
@@ -114,39 +116,24 @@ def linkify_datastore_keys(msg):
     try:
       # Useful for logging, but also causes false positives in the search, we
       # find and use log requests instead of real requests.
-      # logging.debug('Linkifying datastore key: %s', match.group(2))
+      # logger.debug(f'Linkifying datastore key: {'match.group(2)}')
       key = ndb.Key(urlsafe=match.group(2))
-      tokens = [(kind, '%s:%s' % ('id' if isinstance(id, int) else 'name', id))
+      tokens = [(kind, f"{'id' if isinstance(id, int) else 'name'}:{id}")
                 for kind, id in key.pairs()]
-      key_str = '0/|' + '|'.join('%d/%s|%d/%s' % (len(kind), kind, len(id), id)
+      key_str = '0/|' + '|'.join(f'{len(kind)}/{kind}|{len(id)}/{id}'
                                  for kind, id in tokens)
       key_quoted = urllib.parse.quote(urllib.parse.quote(key_str, safe=''), safe='')
-      html = "%s<a title='%s' href='https://console.cloud.google.com/datastore/entities;kind=%s;ns=__$DEFAULT$__/edit;key=%s?project=%s'>%s...</a>%s" % (
-        match.group(1), match.group(2), key.kind(), key_quoted, APP_ID,
-        match.group(3), match.group(4))
-      logging.debug('Returning %s', html)
+      html = f"{match.group(1)}<a title='{match.group(2)}' href='https://console.cloud.google.com/datastore/entities;kind={key.kind()};ns=__$DEFAULT$__/edit;key={key_quoted}?project={APP_ID}'>{match.group(3)}...</a>{match.group(4)}"
+      logger.debug(f'Returning {html}')
       return html
     except BaseException:
-      # logging.debug("Couldn't linkify candidate datastore key.")   # too noisy
+      # logger.debug("Couldn't linkify candidate datastore key.")   # too noisy
       return match.group(0)
 
   return DATASTORE_KEY_RE.sub(linkify_key, msg)
 
 
-def utcfromtimestamp(val):
-    """Wrapper for datetime.utcfromtimestamp that returns HTTP 400 on overflow.
-
-    ...specifically, if datetime.utcfromtimestamp raises OverflowError because
-    the timestamp is greater than the platform's time_t can hold.
-    https://docs.python.org/3.9/library/datetime.html#datetime.datetime.utcfromtimestamp
-    """
-    try:
-      return datetime.datetime.utcfromtimestamp(val)
-    except OverflowError:
-      return error(f'start_time too big: {val}')
-
-
-def log():
+def log(module=None):
     """Flask view that searches for and renders app logs for an HTTP request.
 
     URL parameters:
@@ -161,34 +148,41 @@ def log():
       @cache.cached(600)
       def log():
         return logs.log()
+
+    Args:
+      module: str, App Engine module to search. Defaults to all.
     """
     start_time = flask_util.get_required_param('start_time')
     if not util.is_float(start_time):
-      return error("Couldn't convert start_time to float: %r" % start_time)
+      return error(f"Couldn't convert start_time to float: {start_time!r}")
 
     start_time = float(start_time)
     if start_time < MIN_START_TIME:
-      return error("start_time must be >= %s" % MIN_START_TIME)
+      return error(f'start_time must be >= {MIN_START_TIME}')
+    elif start_time > MAX_START_TIME:
+      return error(f'start_time must be <= {MAX_START_TIME}')
 
     client = Client()
-    project = 'projects/%s' % APP_ID
+    project = f'projects/{APP_ID}'
     key = urllib.parse.unquote_plus(flask_util.get_required_param('key'))
 
     # first, find the individual log message to get the trace id
-    timestamp_filter = 'timestamp>="%s" timestamp<="%s"' % (
-      utcfromtimestamp(start_time - 60).isoformat() + 'Z',
-      utcfromtimestamp(start_time + 120).isoformat() + 'Z')
-    query = 'logName="%s/logs/app" jsonPayload.message:"%s" %s' % (
-      project, key, timestamp_filter)
-    logging.info('Searching logs with: %s', query)
+    utcfromtimestamp = datetime.utcfromtimestamp
+    timestamp_filter = (
+      f"timestamp>=\"{utcfromtimestamp(start_time - 60).isoformat() + 'Z'}\" "
+      f"timestamp<=\"{utcfromtimestamp(start_time + 120).isoformat() + 'Z'}\"")
+    query = f'logName="{project}/logs/python" textPayload:"{key}" {timestamp_filter}'
+    if module:
+      query += f' resource.labels.module_id="{module}"'
+    logger.info(f'Searching logs with: {query}')
     try:
       # https://googleapis.dev/python/logging/latest/client.html#google.cloud.logging_v2.client.Client.list_entries
       log = next(iter(client.list_entries(filter_=query, page_size=1)))
     except StopIteration:
-      logging.info('No log found!')
+      logger.info('No log found!')
       return 'No log found!', 404
 
-    logging.info('Got insert id %s trace %s', log.insert_id, log.trace)
+    logger.info(f'Got insert id {log.insert_id} trace {log.trace}')
 
     # now, print all logs with that trace
     resp = """\
@@ -196,19 +190,26 @@ def log():
 <body style="font-family: monospace; white-space: pre">
 """
 
-    query = 'logName="%s/logs/app" trace="%s" resource.type="gae_app" %s' % (
-      project, log.trace, timestamp_filter)
-    logging.info('Searching logs with: %s', query)
+    query = f'logName="{project}/logs/python" trace="{log.trace}" resource.type="gae_app" {timestamp_filter}'
+    logger.info(f'Searching logs with: {query}')
 
     # sanitize and render each line
     for log in client.list_entries(filter_=query, page_size=1000):
-      msg = log.payload.get('message')
-      if msg:
-        msg = linkify_datastore_keys(util.linkify(html.escape(
-          msg if msg.startswith('Created by this poll:') else sanitize(msg),
-          quote=False)))
-        resp += '%s %s %s<br />' % (
-          log.severity[0], log.timestamp, msg.replace('\n', '<br />'))
+      # payload is a union that can be string, JSON, or protobuf
+      # https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry#FIELDS.oneof_payload
+      msg = log.payload
+      if not msg:
+        continue
+      elif isinstance(msg, (dict, list)):
+        msg = json_dumps(msg, indent=2)
+      else:
+        msg = str(msg)
+
+      msg = linkify_datastore_keys(util.linkify(html.escape(
+        msg if msg.startswith('Created by this poll:') else sanitize(msg),
+        quote=False)))
+      resp += '%s %s %s<br />' % (
+        log.severity[0], log.timestamp, msg.replace('\n', '<br />'))
 
     resp += '</body>\n</html>'
     return resp, {'Content-Type': 'text/html; charset=utf-8'}
