@@ -7,8 +7,9 @@ import collections
 from collections.abc import Iterator
 import contextlib
 import base64
-import datetime
+from datetime import datetime, timedelta, timezone
 import http.client
+import humanize
 import inspect
 import logging
 import mimetypes
@@ -112,7 +113,7 @@ logger = logging.getLogger(__name__)
 # set with set_user_agent()
 user_agent = 'webutil (https://github.com/snarfed/webutil)'
 
-EPOCH = datetime.datetime.utcfromtimestamp(0)
+EPOCH = datetime.fromtimestamp(0, timezone.utc)
 EPOCH_ISO = EPOCH.isoformat()
 # from https://stackoverflow.com/a/53140944/186123
 ISO8601_DURATION_RE = re.compile(
@@ -406,26 +407,30 @@ def favicon_for_url(url):
 
 FULL_HOST_RE = re.compile(HOST_RE + '$')
 
-def domain_from_link(url):
+def domain_from_link(url, minimize=True):
   """Extracts and returns the meaningful domain from a URL.
 
-  Strips www., mobile., and m. from the beginning of the domain.
-
   Args:
     url: string
+    minimize: bool; if true, strips www., mobile., and m. subdomains from the
+      beginning of the domain
 
   Returns:
     string
   """
-  parsed = urlparse(url)
-  if not parsed.hostname and '//' not in url:
-    parsed = urlparse('http://' + url)
+  try:
+    parsed = urlparse(url)
+    if not parsed.hostname and '//' not in url:
+      parsed = urlparse('http://' + url)
+  except ValueError:
+    return None
 
   domain = parsed.hostname
-  if domain:
+  if domain and minimize:
     for subdomain in ('www.', 'mobile.', 'm.'):
       if domain.startswith(subdomain):
         domain = domain[len(subdomain):]
+
   if domain and FULL_HOST_RE.match(domain):
     return domain
 
@@ -571,6 +576,12 @@ def base_url(url):
   return urllib.parse.urljoin(url, ' ')[:-1] if url else None
 
 
+def is_web(url):
+  """Returns True if the argument is an http or https URL, False otherwise."""
+  return isinstance(url, str) and (url.startswith('http://') or
+                                   url.startswith('https://'))
+
+
 def extract_links(text):
   """Returns a list of unique string URLs in the given text.
 
@@ -709,7 +720,7 @@ def pretty_link(url, text=None, keep_host=True, glyphicon=None, attrs=None,
   else:
     # use shortened version of URL as link text
     parsed = urlparse(url)
-    text = url[len(parsed.scheme) + 3:]  # strip scheme and ://
+    text = url.removeprefix(parsed.scheme + '://')
     host_len = len(parsed.netloc)
     if (keep_host and not parsed.params and not parsed.query and not parsed.fragment):
       text = text.strip('/')  # drop trailing slash
@@ -726,6 +737,7 @@ def pretty_link(url, text=None, keep_host=True, glyphicon=None, attrs=None,
     except ValueError:
       pass
 
+  full_text = text
   if max_length and len(text) > max_length:
     text = text[:max_length] + '...'
 
@@ -736,29 +748,15 @@ def pretty_link(url, text=None, keep_host=True, glyphicon=None, attrs=None,
   attr_str = (''.join(f'{attr}="{val}" ' for attr, val in list(attrs.items()))
               if attrs else '')
   target = 'target="_blank" ' if new_tab else ''
-  return ('<a %s%shref="%s">%s</a>' %
-          (attr_str, target,
+  return ('<a %s%s%shref="%s">%s</a>' %
+          (f'title="{full_text}" ' if text.endswith('...') else '',
+           attr_str, target,
            # not using urllib.parse.quote because it quotes a ton of chars we
            # want to pass through, including most unicode chars
            url.replace('<', '%3C').replace('>', '%3E'),
            escaped_text))
 
 
-class SimpleTzinfo(datetime.tzinfo):
-  """A simple, DST-unaware tzinfo subclass.
-  """
-
-  offset = datetime.timedelta(0)
-
-  def utcoffset(self, dt):
-    return self.offset
-
-  def dst(self, dt):
-    return datetime.timedelta(0)
-
-UTC = SimpleTzinfo()
-
-
 TIMEZONE_OFFSET_RE = re.compile(r'[+-]\d{2}:?\d{2}$')
 
 def parse_iso8601(val):
@@ -783,23 +781,23 @@ def parse_iso8601(val):
   zone = TIMEZONE_OFFSET_RE.search(val)
 
   if zone:
-    offset = zone.group()
-    val = val[:-len(offset)]
-    tz = SimpleTzinfo()
-    tz.offset = (datetime.datetime.strptime(offset[1:].replace(':', ''), '%H%M') -
-                 datetime.datetime.strptime('', ''))
-    if offset[0] == '-':
-      tz.offset = -tz.offset
+    offset_str = zone.group()
+    val = val[:-len(offset_str)]
+    offset = (datetime.strptime(offset_str[1:].replace(':', ''), '%H%M') -
+              datetime.strptime('', ''))
+    if offset_str[0] == '-':
+      offset = -offset
+    tz = timezone(offset)
   elif val[-1] == 'Z':
     val = val[:-1]
-    tz = UTC
+    tz = timezone.utc
 
   # fractional seconds are optional. add them if they're not already there to
   # make strptime parsing below easier.
   if '.' not in val:
     val += '.0'
 
-  return datetime.datetime.strptime(val, '%Y-%m-%d %H:%M:%S.%f').replace(tzinfo=tz)
+  return datetime.strptime(val, '%Y-%m-%d %H:%M:%S.%f').replace(tzinfo=tz)
 
 
 def parse_iso8601_duration(input):
@@ -829,9 +827,9 @@ def parse_iso8601_duration(input):
     val = match.group(i)
     return int(val[:-1]) if val else 0
 
-  return datetime.timedelta(weeks=g(3),
-                            days=365 * g(1) + 30 * g(2) + g(4),
-                            hours=g(6), minutes=g(7), seconds=g(8))
+  return timedelta(weeks=g(3),
+                   days=365 * g(1) + 30 * g(2) + g(4),
+                   hours=g(6), minutes=g(7), seconds=g(8))
 
 
 def to_iso8601_duration(input):
@@ -850,8 +848,8 @@ def to_iso8601_duration(input):
 
   Raises: :class:`TypeError` if delta is not a :class:`datetime.timedelta`
   """
-  if not isinstance(input, datetime.timedelta):
-    raise TypeError(f'Expected datetime.timedelta, got {input.__class__}')
+  if not isinstance(input, timedelta):
+    raise TypeError(f'Expected timedelta, got {input.__class__}')
 
   return f'P{input.days}DT{input.seconds}S'
 
@@ -879,7 +877,7 @@ def maybe_timestamp_to_rfc3339(input):
   Assumes UNIX timestamps are always UTC. (They're generally supposed to be.)
   """
   try:
-    dt = datetime.datetime.utcfromtimestamp(float(input)).replace(tzinfo=UTC)
+    dt = datetime.utcfromtimestamp(float(input)).replace(tzinfo=timezone.utc)
     return dt.isoformat('T', 'milliseconds' if dt.microsecond else 'seconds')
   except (ValueError, TypeError):
     return input
@@ -915,10 +913,22 @@ def as_utc(input):
   if not input.tzinfo:
     return input
 
-  utc = input - input.tzinfo.utcoffset(False)
+  utc = input - input.tzinfo.utcoffset(None)
   return utc.replace(tzinfo=None)
 
 
+def naturaltime(val, when=None, **kwargs):
+  """Wrapper for humanize.naturaltime that handles timezone-aware datetimes.
+
+  ...since humanize currently doesn't. :(
+  https://github.com/python-humanize/humanize/issues/17
+  """
+  val = val.replace(tzinfo=None)
+  if when is not None:
+    when = when.replace(tzinfo=None)
+  return humanize.naturaltime(val, when=when, **kwargs)
+
+
 def ellipsize(str, words=14, chars=140):
   """Truncates and ellipsizes str if it's longer than words or chars.
 
@@ -1258,6 +1268,9 @@ def interpret_http_exception(exception):
   elif ((requests and isinstance(e, requests.HTTPError)) or
         (prawcore and isinstance(e, prawcore.exceptions.ResponseException))):
     code = e.response.status_code
+    # TODO: this (and same below) can raise if the body was already read:
+    # 'RuntimeError: The content for this response was already consumed'
+    # https://console.cloud.google.com/errors/detail/CKuWp8f18s3g0gE;time=P30D?project=brid-gy
     body = e.response.text
 
   elif tumblpy and isinstance(e, tumblpy.TumblpyError):
@@ -1415,7 +1428,10 @@ def is_connection_failure(exception):
     ]
 
   if urllib3:
-    types += [urllib3.exceptions.HTTPError]
+    types += [
+      urllib3.exceptions.HTTPError,
+      urllib3.exceptions.ReadTimeoutError,
+    ]
 
   msg = str(exception)
   if (isinstance(exception, tuple(types)) or
@@ -1427,10 +1443,7 @@ def is_connection_failure(exception):
       'Connection closed unexpectedly' in msg or
       'Max retries exceeded' in msg
      ):
-    # TODO: exc_info might not be for exception, e.g. if the json_loads() in
-    # interpret_http_exception() fails. need to pass through the whole
-    # sys.exc_info() tuple here, not just the exception object.
-    logger.info(f'Connection failure: {exception}', stack_info=True)
+    logger.info(f'Connection failure: {exception}', stack_info=False)
     return True
 
   return False
@@ -1559,8 +1572,8 @@ def requests_fn(fn):
       (HTTP 400), connection failures and HTTP 4xx and 5xx result in
       :class:`werkzeug.exceptions.BadGateway` (HTTP 502).
   """
-  def call(url, *args, **kwargs):
-    logger.info(f'requests.{fn} {url} {_prune(kwargs)}')
+  def call(url, session=None, *args, **kwargs):
+    logger.info(f'{session or "requests"}.{fn} {url} {_prune(kwargs)}')
 
     gateway = kwargs.pop('gateway', None)
     kwargs.setdefault('timeout', HTTP_TIMEOUT)
@@ -1573,7 +1586,7 @@ def requests_fn(fn):
 
     try:
       # use getattr so that stubbing out with mox still works
-      resp = getattr(requests, fn)(url, *args, **kwargs)
+      resp = getattr((session or requests), fn)(url, *args, **kwargs)
       if gateway:
         logger.info(f'Received {resp.status_code}: {"" if resp.ok else resp.text[:500]}')
         resp.raise_for_status()
@@ -1630,7 +1643,10 @@ def requests_fn(fn):
         if gateway:
           resp.raise_for_status()
 
-    logger.info(f'Received {resp.status_code}')
+    msg = f'Received {resp.status_code}'
+    if resp.status_code // 100 == 3:
+      msg += f' {resp.headers.get("Location") or "no Location header"}'
+    logger.info(msg)
 
     return resp
 
@@ -1963,6 +1979,46 @@ def parse_mf2(input, url=None, id=None):
   return mf2py.parse(url=url, doc=input, img_with_alt=True)
 
 
+def parse_http_equiv(content):
+  """Parses the value in the http_equiv meta field and returns the url.
+
+  Args:
+    content: str, http_equiv content string https://www.w3.org/TR/WCAG20-TECHS/H76.html#procedure
+
+  Returns: str, empty if content format is incorrect
+  """
+  split = content.rpartition('URL=')
+  if not split[1]: # If URL= is not in the string return an empty string
+    return ''
+
+  return split[2].strip("'")
+
+
+def fetch_http_equiv(input, **kwargs):
+  """Fetches http_equiv meta tag, if available.
+
+  Args:
+    input: unicode HTML string, :class:`bs4.BeautifulSoup`, or
+      :class:`requests.Response`
+
+  Returns: str, empty if not available or a url if available
+  """
+  if not isinstance(input, (bs4.BeautifulSoup, bs4.Tag)):
+    input = parse_html(input)
+
+  element = input.find('meta', attrs={'http-equiv': 'refresh'})
+
+  if not element:
+    return ''
+
+  refresh_content = element.get('content')
+
+  if not refresh_content:
+    return ''
+
+  return parse_http_equiv(refresh_content)
+
+
 def fetch_mf2(url, get_fn=requests_get, gateway=False, **kwargs):
   """Fetches an HTML page over HTTP, parses it, and returns its microformats2.
 
