@@ -20,15 +20,20 @@ LINK_HEADER_RE = re.compile(
 Endpoint = namedtuple('Endpoint', ('endpoint', 'response'))
 
 
-def discover(url, **requests_kwargs):
+def discover(url, follow_meta_refresh=False, **requests_kwargs):
   """Discovers a URL's webmention endpoint.
 
+  Follows up to 30 HTTP 3xx redirects, and at most one client-side HTML meta
+  http-equiv=refresh redirects.
+
   Args:
     url: str
+    follow_meta_refresh: bool, whether to follow client side redirects in HTML
+      meta http-equiv=refresh tags
     requests_kwargs: passed to :meth:`requests.post`
 
   Returns: :class:`Endpoint`. If no endpoint is discovered, the endpoint
-  attribute will be None.
+    attribute will be None.
 
   Raises: :class:`ValueError` on bad URL, :class:`requests.HTTPError` on failure
   """
@@ -67,6 +72,13 @@ def discover(url, **requests_kwargs):
       logger.debug(f'Webmention discovery: got endpoint in tag: {endpoint}')
       return Endpoint(endpoint, resp)
 
+  http_equiv = util.fetch_http_equiv(soup)
+  if http_equiv: # else implicit break out and continue like normal
+    endpoint = util.fragmentless(urljoin(url, http_equiv))
+    if follow_meta_refresh and url != endpoint:
+      logger.debug(f'Webmention discovery: following http_equiv: {endpoint}')
+      return discover(endpoint, **requests_kwargs)
+
   logger.debug('Webmention discovery: no endpoint in headers or HTML')
   return Endpoint(None, resp)
 
@@ -100,6 +112,6 @@ def send(endpoint, source, target, **requests_kwargs):
     logger.debug(f'webmention send: got {e.__class__.__name__}')
     raise
 
-  logger.debug(f'webmention send: got HTTP {resp.status_code}')
+  logger.debug(f'webmention send: got HTTP {resp.status_code} {resp.headers.get("Location", "")}')
   resp.raise_for_status()
   return resp
